// Blue Lagoon 1.0
// Script: RR
// Concept: Bubble

// Main entry point for random map script
void main(void)
{
	// Text
	rmSetStatusText("",1.0);

	// Set size
	int playerTiles=7500;
	int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
	rmSetMapSize(size, size);
	rmEchoInfo("Map size="+size+"m x "+size+"m");

	// Init map
	rmSetSeaLevel(-1.0);
	rmTerrainInitialize("SandA", 0.0);
	// rmSetLightingSet("Default");

	// Define clssses
	int classCliff=rmDefineClass("cliff");
	int classPond=rmDefineClass("pond");
	int classPlayer=rmDefineClass("player");
	int classForest=rmDefineClass("forest");
	rmDefineClass("starting settlement");

	// Constraints
	// General
	int farEdgeConstraint=rmCreateBoxConstraint("far edge constraint", rmXTilesToFraction(12), rmZTilesToFraction(12), 1.0-rmXTilesToFraction(12), 1.0-rmZTilesToFraction(12));
	int edgeConstraint=rmCreateBoxConstraint("edge constraint", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8));
	int shortEdgeConstraint=rmCreateBoxConstraint("short edge constraint", rmXTilesToFraction(4), rmZTilesToFraction(4), 1.0-rmXTilesToFraction(4), 1.0-rmZTilesToFraction(4));
	int shortAvoidAll=rmCreateTypeDistanceConstraint("short avoid all", "all", 8.0);
	int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 1.0);

	// Terrain
	int farCliffConstraint=rmCreateClassDistanceConstraint("long avoid cliff", rmClassID("cliff"), 25.0);
	int mediumCliffConstraint=rmCreateClassDistanceConstraint("medium avoid cliff", rmClassID("cliff"), 10.0);
	int shortCliffConstraint=rmCreateClassDistanceConstraint("short avoid cliff", rmClassID("cliff"), 5.0);
	int cliffPlayerConstraint=rmCreateClassDistanceConstraint("cliffs stay away from players", classPlayer, 5.0);
	int cliffEdgeConstraint=rmCreateBoxConstraint("cliffs avoid edge of the map", rmXTilesToFraction(32), rmZTilesToFraction(32), 1.0-rmXTilesToFraction(32), 1.0-rmZTilesToFraction(32));
	int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "land", false, 4.0);
	int pondConstraint=rmCreateClassDistanceConstraint("pond vs. pond", rmClassID("pond"), 25.0);
	int mediumPondConstraint=rmCreateClassDistanceConstraint("medium pond constraint", rmClassID("pond"), 8.0);
	int shortPondConstraint=rmCreateClassDistanceConstraint("short pond constraint", rmClassID("pond"), 3.0);

	// Settlements
	int startingSettleConstraint=rmCreateClassDistanceConstraint("avoid starting settlement", rmClassID("starting settlement"), 30.0);
	int shortStartingSettleConstraint=rmCreateClassDistanceConstraint("short avoid starting settlement", rmClassID("starting settlement"), 10.0);
	int avoidSettlement=rmCreateTypeDistanceConstraint("avoid TCs", "AbstractSettlement", 50.0);
	int shortAvoidSettlement=rmCreateTypeDistanceConstraint("short avoid TCs", "AbstractSettlement", 20.0);
	int mediumAvoidSettlement=rmCreateTypeDistanceConstraint("medium avoid TCs", "AbstractSettlement", 45.0); // For ponds, try to lower value without having missing tcs.

	// Gold
	int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 8.0);
	int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
	int farAvoidGold=rmCreateTypeDistanceConstraint("far avoid gold", "gold", 45.0);

	// Food
	int avoidHunt=rmCreateTypeDistanceConstraint("avoid huntable", "huntable", 35.0);
	int shortAvoidFood=rmCreateTypeDistanceConstraint("short avoid huntable", "food", 8.0);
	int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 40.0);
	int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 40.0);

	// Wood
	int shortForestConstraint=rmCreateClassDistanceConstraint("short avoid forest", rmClassID("forest"), 5.0);
	int mediumForestConstraint=rmCreateClassDistanceConstraint("medium avoid forest", rmClassID("forest"), 12.5);
	int farForestConstraint=rmCreateClassDistanceConstraint("far avoid forest", rmClassID("forest"), 20.0);
	int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);

	// Buildings
	int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 15.0);
	int avoidTower=rmCreateTypeDistanceConstraint("avoid tower", "tower", 25.0);
	int shortAvoidTower=rmCreateTypeDistanceConstraint("short avoid tower", "tower", 10.0);

	// Text
	rmSetStatusText("",0.9);

	// Define objects
	// Starting settlement
	int startingSettlementID=rmCreateObjectDef("Starting settlement");
	rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
	rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
	rmSetObjectDefMinDistance(startingSettlementID, 0.0);
	rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

	// Towers
	int startingTowerID=rmCreateObjectDef("starting tower");
	rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
	rmSetObjectDefMinDistance(startingTowerID, 22.0);
	rmSetObjectDefMaxDistance(startingTowerID, 24.0);
	rmAddObjectDefConstraint(startingTowerID, avoidTower);

	// Gold
	int startingGoldID=rmCreateObjectDef("starting gold");
	rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
	rmSetObjectDefMinDistance(startingGoldID, 20.0);
	rmSetObjectDefMaxDistance(startingGoldID, 25.0);
	rmAddObjectDefConstraint(startingGoldID, shortAvoidAll);
	rmAddObjectDefConstraint(startingGoldID, shortAvoidTower);

	// Hunt
	int startingGiraffeID=rmCreateObjectDef("starting giraffes");
	rmAddObjectDefItem(startingGiraffeID, "Giraffe", 4, 3.0);
	rmSetObjectDefMinDistance(startingGiraffeID, 27.5);
	rmSetObjectDefMaxDistance(startingGiraffeID, 27.5);
	rmAddObjectDefConstraint(startingGiraffeID, shortAvoidAll);

	// Berries
	int startingFoodID=rmCreateObjectDef("close food");
	rmAddObjectDefItem(startingFoodID, "berry bush", rmRandInt(5,7), 4.0);
	rmSetObjectDefMinDistance(startingFoodID, 20.0);
	rmSetObjectDefMaxDistance(startingFoodID, 25.0);
	rmAddObjectDefConstraint(startingFoodID, shortAvoidAll);
	rmAddObjectDefConstraint(startingFoodID, shortAvoidGold);

	// Pigs
	int startingPigsID=rmCreateObjectDef("close pigs");
	rmAddObjectDefItem(startingPigsID, "pig", 3, 2.0);
	rmSetObjectDefMinDistance(startingPigsID, 25.0);
	rmSetObjectDefMaxDistance(startingPigsID, 30.0);
	rmAddObjectDefConstraint(startingPigsID, shortAvoidAll);

	// Straggler trees
	int stragglerTreeID=rmCreateObjectDef("straggler tree");
	rmAddObjectDefItem(stragglerTreeID, "savannah tree", 1, 0.0);
	rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
	rmSetObjectDefMaxDistance(stragglerTreeID, 20.0);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidGold);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidAll);

	// Medium objects
	// Medium gold
	int mediumGoldID=rmCreateObjectDef("medium gold");
	rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(mediumGoldID, 50.0);
	rmSetObjectDefMaxDistance(mediumGoldID, 65.0);
	rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
	rmAddObjectDefConstraint(mediumGoldID, edgeConstraint);
	rmAddObjectDefConstraint(mediumGoldID, mediumCliffConstraint);
	rmAddObjectDefConstraint(mediumGoldID, avoidGold);
	rmAddObjectDefConstraint(mediumGoldID, mediumPondConstraint);

	// Medium zebras
	int mediumZebraID=rmCreateObjectDef("medium zebras");
	rmAddObjectDefItem(mediumZebraID, "zebra", rmRandInt(4,5), 3.0);
	rmSetObjectDefMinDistance(mediumZebraID, 55.0);
	rmSetObjectDefMaxDistance(mediumZebraID, 65.0);
	rmAddObjectDefConstraint(mediumZebraID, shortAvoidAll);
	rmAddObjectDefConstraint(mediumZebraID, edgeConstraint);
	rmAddObjectDefConstraint(mediumZebraID, avoidHunt);
	rmAddObjectDefConstraint(mediumZebraID, shortCliffConstraint);
	rmAddObjectDefConstraint(mediumZebraID, startingSettleConstraint);
	rmAddObjectDefConstraint(mediumZebraID, shortPondConstraint);

	// Medium elephants
	int mediumElephantID=rmCreateObjectDef("medium elephants");
	rmAddObjectDefItem(mediumElephantID, "elephant", 2, 3.0);
	rmSetObjectDefMinDistance(mediumElephantID, 60.0);
	rmSetObjectDefMaxDistance(mediumElephantID, 70.0);
	rmAddObjectDefConstraint(mediumElephantID, shortAvoidAll);
	rmAddObjectDefConstraint(mediumElephantID, edgeConstraint);
	rmAddObjectDefConstraint(mediumElephantID, avoidHunt);
	rmAddObjectDefConstraint(mediumElephantID, shortCliffConstraint);
	rmAddObjectDefConstraint(mediumElephantID, startingSettleConstraint);
	rmAddObjectDefConstraint(mediumElephantID, shortPondConstraint);

	// Medium pigs
	int mediumPigsID=rmCreateObjectDef("medium pigs");
	rmAddObjectDefItem(mediumPigsID, "pig", 2, 2.0);
	rmSetObjectDefMinDistance(mediumPigsID, 55.0);
	rmSetObjectDefMaxDistance(mediumPigsID, 65.0);
	rmAddObjectDefConstraint(mediumPigsID, avoidHerdable);
	rmAddObjectDefConstraint(mediumPigsID, shortAvoidAll);
	rmAddObjectDefConstraint(mediumPigsID, edgeConstraint);
	rmAddObjectDefConstraint(mediumPigsID, shortCliffConstraint);
	rmAddObjectDefConstraint(mediumPigsID, shortPondConstraint);

	// Predators
	int predatorID=rmCreateObjectDef("predator");
	rmAddObjectDefItem(predatorID, "hyena", 3, 4.0);
	rmSetObjectDefMinDistance(predatorID, 70.0);
	rmSetObjectDefMaxDistance(predatorID, 70.0);
	rmAddObjectDefConstraint(predatorID, shortAvoidAll);
	rmAddObjectDefConstraint(predatorID, edgeConstraint);
	rmAddObjectDefConstraint(predatorID, avoidPredator);
	rmAddObjectDefConstraint(predatorID, shortCliffConstraint);
	rmAddObjectDefConstraint(predatorID, playerConstraint);
	rmAddObjectDefConstraint(predatorID, shortPondConstraint);

	// Far objects
	// Far gold
	int farGoldID=rmCreateObjectDef("far gold");
	rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(farGoldID, 60.0);
	rmSetObjectDefMaxDistance(farGoldID, 100.0);
	rmAddObjectDefConstraint(farGoldID, edgeConstraint);
	rmAddObjectDefConstraint(farGoldID, farAvoidGold);
	rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
	rmAddObjectDefConstraint(farGoldID, mediumCliffConstraint);
	rmAddObjectDefConstraint(farGoldID, startingSettleConstraint);
	rmAddObjectDefConstraint(farGoldID, shortPondConstraint);

	// Far zebras
	int farZebraID=rmCreateObjectDef("far zebras");
	rmAddObjectDefItem(farZebraID, "zebra", rmRandInt(5,6), 3.0);
	rmSetObjectDefMinDistance(farZebraID, 70.0);
	rmSetObjectDefMaxDistance(farZebraID, 80.0);
	rmAddObjectDefConstraint(farZebraID, shortAvoidAll);
	rmAddObjectDefConstraint(farZebraID, edgeConstraint);
	rmAddObjectDefConstraint(farZebraID, avoidHunt);
	rmAddObjectDefConstraint(farZebraID, shortCliffConstraint);
	rmAddObjectDefConstraint(farZebraID, startingSettleConstraint);
	rmAddObjectDefConstraint(farZebraID, shortPondConstraint);

	// Far elephants
	int farElephantID=rmCreateObjectDef("far elephants");
	rmAddObjectDefItem(farElephantID, "elephant", 2, 3.0);
	rmSetObjectDefMinDistance(farElephantID, 70.0);
	rmSetObjectDefMaxDistance(farElephantID, 100.0);
	rmAddObjectDefConstraint(farElephantID, shortAvoidAll);
	rmAddObjectDefConstraint(farElephantID, edgeConstraint);
	rmAddObjectDefConstraint(farElephantID, avoidHunt);
	rmAddObjectDefConstraint(farElephantID, shortCliffConstraint);
	rmAddObjectDefConstraint(farElephantID, startingSettleConstraint);
	rmAddObjectDefConstraint(farElephantID, shortPondConstraint);

	// Random Trees
	int randomTreeID=rmCreateObjectDef("random tree");
	rmAddObjectDefItem(randomTreeID, "savannah tree", 1, 0.0);
	rmSetObjectDefMinDistance(randomTreeID, 0.0);
	rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
	rmAddObjectDefConstraint(randomTreeID, shortAvoidAll);
	rmAddObjectDefConstraint(randomTreeID, shortCliffConstraint);
	rmAddObjectDefConstraint(randomTreeID, shortStartingSettleConstraint);

	// Relics
	int relicID=rmCreateObjectDef("relics");
	rmAddObjectDefItem(relicID, "relic", 1, 0.0);
	rmSetObjectDefMinDistance(relicID, 70.0);
	rmSetObjectDefMaxDistance(relicID, rmXFractionToMeters(0.3));
	rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 50.0));
	rmAddObjectDefConstraint(relicID, shortAvoidAll);
	rmAddObjectDefConstraint(relicID, shortCliffConstraint);
	rmAddObjectDefConstraint(relicID, edgeConstraint);
	rmAddObjectDefConstraint(relicID, startingSettleConstraint);

	// Birds
	int farhawkID=rmCreateObjectDef("far hawks");
	rmAddObjectDefItem(farhawkID, "vulture", 1, 0.0);
	rmSetObjectDefMinDistance(farhawkID, 0.0);
	rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

	// Text
	rmSetStatusText("",0.8);

	// Player placement
	rmSetTeamSpacingModifier(0.9);
	rmPlacePlayersCircular(0.39, 0.39, rmDegreesToRadians(5.0));

	// Set up player areas
	float playerFraction=rmAreaTilesToFraction(1400);
	for(i=1; <cNumberPlayers)
	{
		int id=rmCreateArea("Player"+i);
		rmSetPlayerArea(i, id);
		rmSetAreaSize(id, playerFraction, playerFraction);
		rmAddAreaToClass(id, classPlayer);
		rmSetAreaTerrainType(id, "GrassDirt50");
		rmAddAreaTerrainLayer(id, "GrassDirt75", 6, 10);
		rmSetAreaWarnFailure(id, false);
		rmSetAreaCoherence(id, 0.6);
		rmSetAreaLocPlayer(id, i);
	}

	rmBuildAllAreas();

	// Text
	rmSetStatusText("",0.7);

	// Place settlements
	rmPlaceObjectDefPerPlayer(startingSettlementID, true);
	int placementInt=0;

	// Settlement 1
	if(cNumberNonGaiaPlayers < 3)
		id=rmAddFairLoc("Settlement", false, true, 60, 80, 40, 16);
	else if(cNumberNonGaiaPlayers < 5)
	{
		if(rmRandFloat(0,1)<0.67)

			id=rmAddFairLoc("Settlement", false, true, 60, 85, 40, 16);
		else
		{
			id=rmAddFairLoc("Settlement", false, false, 60, 85, 40, 16);
			placementInt=1;
		}
	}
	else
		id=rmAddFairLoc("Settlement", false, true, 60, 85, 40, 16);

	if(rmPlaceFairLocs())
	{
		id=rmCreateObjectDef("close settlement");
		rmAddObjectDefItem(id, "Settlement", 1, 0.0);
		for(i=1; <cNumberPlayers)
		{
			for(j=0; <rmGetNumberFairLocs(i))
			{
				int settleArea = rmCreateArea("close settlement area"+i +j, rmAreaID("Player"+i));
				rmSetAreaLocation(settleArea, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j));
				rmBuildArea(settleArea);
				rmPlaceObjectDefAtAreaLoc(id, i, settleArea);
			}
		}
	}

	rmResetFairLocs();

	// Text
	rmSetStatusText("",0.6);

	// Settlement 2
	if(cNumberNonGaiaPlayers < 3)
		id=rmAddFairLoc("Settlement", true, false, 75, 80, 50, 50);
	else if(cNumberNonGaiaPlayers < 5)
	{
		if(placementInt==0)
		{
			if(rmRandFloat(0,1)<0.5)
				id=rmAddFairLoc("Settlement", true, false,  75, 90, 70, 40);
			else
				id=rmAddFairLoc("Settlement", true, true,  75, 85, 70, 100);
		}
		else
			id=rmAddFairLoc("Settlement", true, true,  75, 90, 70, 100);
	}
	else
	{
		if(rmRandFloat(0,1)<0.5)
			id=rmAddFairLoc("Settlement", true, false,  75, 90, 70, 100);
		else
			id=rmAddFairLoc("Settlement", true, true,  75, 90, 70, 120);
	}
	rmAddFairLocConstraint(id, startingSettleConstraint);
	rmAddFairLocConstraint(id, avoidSettlement);

	if(rmPlaceFairLocs())
	{
		id=rmCreateObjectDef("far settlement");
		rmAddObjectDefItem(id, "Settlement", 1, 0.0);
		for(i=1; <cNumberPlayers)
		{
			for(j=0; <rmGetNumberFairLocs(i))
			{
				int farSettleArea = rmCreateArea("far settlement area"+i +j, rmAreaID("Player"+i));
				rmSetAreaLocation(farSettleArea, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j));
				rmBuildArea(farSettleArea);
				rmPlaceObjectDefAtAreaLoc(id, i, farSettleArea);
			}
		}
	}

	// Text
	rmSetStatusText("",0.5);

	// Ponds
	int numTries=150;
	int placeCount=0;
	for(i=0; <numTries)
	{
		int smallPondID=rmCreateArea("small pond"+i);
		rmSetAreaSize(smallPondID, rmAreaTilesToFraction(400), rmAreaTilesToFraction(400));
		if(cNumberNonGaiaPlayers==2)
			rmSetAreaLocation(smallPondID, rmRandFloat(0.25, 0.75), rmRandFloat(0.25, 0.75));
		else
			rmSetAreaLocation(smallPondID, rmRandFloat(0.15, 0.85), rmRandFloat(0.15, 0.85));
		rmSetAreaWaterType(smallPondID, "Egyptian Nile");
		rmSetAreaCoherence(smallPondID, 1.0);
		rmSetAreaMinBlobs(smallPondID, 1);
		rmSetAreaMaxBlobs(smallPondID, 1);
		rmAddAreaConstraint(smallPondID, pondConstraint);
		rmAddAreaConstraint(smallPondID, playerConstraint);
		rmAddAreaConstraint(smallPondID, mediumAvoidSettlement);
		rmSetAreaWarnFailure(smallPondID, false);
		rmAddAreaToClass(smallPondID, classPond);

		if(rmBuildArea(smallPondID)==true)
		{
			placeCount++;
			if(placeCount>2+1.5*cNumberNonGaiaPlayers)
				break;
		}
	}

	// Text
	rmSetStatusText("",0.4);

	// Texture embellishment
	for(i=1; <cNumberPlayers*25)
	{
		int id1=rmCreateArea("beautification"+i);
		rmSetAreaSize(id1, rmAreaTilesToFraction(75), rmAreaTilesToFraction(150));
		rmSetAreaTerrainType(id1, "GrassDirt75");
		rmAddAreaConstraint(id1, shortAvoidImpassableLand);
		rmSetAreaMinBlobs(id1, 1);
		rmSetAreaMaxBlobs(id1, 5);
		rmSetAreaWarnFailure(id1, false);
		rmSetAreaMinBlobDistance(id1, 30.0);
		rmSetAreaMaxBlobDistance(id1, 40.0);
		rmSetAreaCoherence(id1, 0.6);
		rmBuildArea(id1);
	}

	// Text
	rmSetStatusText("",0.3);

	// Cliffs
	numTries=40;
	placeCount=0;
	for(i=0; <numTries)
	{
		int cliffID=rmCreateArea("cliff"+i);
		rmSetAreaWarnFailure(cliffID, false);
		rmSetAreaSize(cliffID, rmAreaTilesToFraction(200), rmAreaTilesToFraction(220));
		rmSetAreaCliffType(cliffID, "Egyptian");
		rmSetAreaCliffEdge(cliffID, 2, 0.25, 0.0, 1.0, 1);
		rmSetAreaCliffPainting(cliffID, true, true, true, 1.5, true);
		rmSetAreaCliffHeight(cliffID, 4, 0.0, 1.0);
		rmSetAreaCoherence(cliffID, 1.0);
		rmSetAreaHeightBlend(cliffID, 1.0);
		rmAddAreaToClass(cliffID, classCliff);
		rmAddAreaConstraint(cliffID, mediumPondConstraint);
		rmAddAreaConstraint(cliffID, shortEdgeConstraint);
		rmAddAreaConstraint(cliffID, avoidBuildings);
		rmAddAreaConstraint(cliffID, farCliffConstraint);
		rmAddAreaConstraint(cliffID, shortAvoidSettlement);
		rmAddAreaConstraint(cliffID, playerConstraint);

		if(rmBuildArea(cliffID)==true)
		{
			placeCount++;
			if(placeCount>3.5*cNumberNonGaiaPlayers)
				break;
		}
	}

	// Text
	rmSetStatusText("",0.2);

	// Place close objects
	// Towers
	rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

	// Gold
	rmPlaceObjectDefPerPlayer(startingGoldID, false);

	// Hunt
	rmPlaceObjectDefPerPlayer(startingGiraffeID, false);

	// Berries
	rmPlaceObjectDefPerPlayer(startingFoodID, false);

	// Pigs
	rmPlaceObjectDefPerPlayer(startingPigsID, true);

	// Straggler trees
	rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(4,6));

	// Medium objects
	// Gold
	if(cNumberNonGaiaPlayers < 3)
		rmPlaceObjectDefPerPlayer(mediumGoldID, false, 2);
	else
		rmPlaceObjectDefPerPlayer(mediumGoldID, false);

	// Medium hunt
	rmPlaceObjectDefPerPlayer(mediumZebraID, false);
	rmPlaceObjectDefPerPlayer(mediumElephantID, false);

	// Medium pigs
	rmPlaceObjectDefPerPlayer(mediumPigsID, false);

	// Predators
	rmPlaceObjectDefPerPlayer(predatorID, false);

	// Far objects
	// Gold
	if(cNumberNonGaiaPlayers < 3)
		rmPlaceObjectDefPerPlayer(farGoldID, false, 2);
	else if(cNumberNonGaiaPlayers < 5)
		rmPlaceObjectDefPerPlayer(farGoldID, false, 3);
	else
		rmPlaceObjectDefPerPlayer(farGoldID, false, 2);

	// Far hunt
	rmPlaceObjectDefPerPlayer(farZebraID, false);
	rmPlaceObjectDefPerPlayer(farElephantID, false);

	// Relics
	rmPlaceObjectDefPerPlayer(relicID, false);

	// Random trees
	rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 10*cNumberNonGaiaPlayers);

	// Hawks
	rmPlaceObjectDefPerPlayer(farhawkID, false, 2);

	// Text
	rmSetStatusText("",0.1);

	// Forest
	int failCount=0;
	numTries=12*cNumberNonGaiaPlayers;
	for(i=0; <numTries)
	{
		int forestID=rmCreateArea("forest"+i);
		rmSetAreaSize(forestID, rmAreaTilesToFraction(50), rmAreaTilesToFraction(80));
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaForestType(forestID, "savannah forest");
		rmAddAreaConstraint(forestID, forestSettleConstraint);
		rmAddAreaConstraint(forestID, shortAvoidAll);
		rmAddAreaConstraint(forestID, farForestConstraint);
		rmAddAreaConstraint(forestID, mediumCliffConstraint);
		rmAddAreaConstraint(forestID, shortPondConstraint);
		rmAddAreaToClass(forestID, classForest);

		rmSetAreaMinBlobs(forestID, 3);
		rmSetAreaMaxBlobs(forestID, 5);
		rmSetAreaMinBlobDistance(forestID, 20.0);
		rmSetAreaMaxBlobDistance(forestID, 30.0);
		rmSetAreaCoherence(forestID, 0.0);

		if(rmBuildArea(forestID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3)
				break;
		}
		else
			failCount=0;
	}

	// Text
	rmSetStatusText("",0.01);

	// Notify players
	if(cNumberTeams > 2)
	{
		rmCreateTrigger("team notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
	else if(cNumberNonGaiaPlayers > 6)
	{
		rmCreateTrigger("player notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
}