// Frozen Wastes 1.0
// Script: RR
// Concept: Bubble

// Main entry point for random map script
void main(void)
{
	// Text
	rmSetStatusText("",1.0);

	// Set size
	int playerTiles = 10200-300*cNumberNonGaiaPlayers;
	int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
	rmEchoInfo("Map size="+size+"m x "+size+"m");
	rmSetMapSize(size, size);

	// Init map.
	rmTerrainInitialize("IceA");
	rmSetLightingSet("Alfheim");

	// Define classes
	int classForest=rmDefineClass("forest");
	rmDefineClass("starting settlement");
	rmDefineClass("smallCenter");

	// Generic stuff
	// Build continent
	int centerID=rmCreateArea("center");
	rmSetAreaSize(centerID, 0.525, 0.525);
	rmSetAreaLocation(centerID, 0.5, 0.5);
	if(cNumberNonGaiaPlayers < 5)
		rmSetAreaCoherence(centerID, 0.0);
	else
		rmSetAreaCoherence(centerID, 0.4);
	rmSetAreaBaseHeight(centerID, 2.0);
	rmSetAreaTerrainType(centerID, "SnowA");
	rmSetAreaSmoothDistance(centerID, 30);
	rmSetAreaHeightBlend(centerID, 2);
	rmAddAreaConstraint(centerID, rmCreateBoxConstraint("center borders", rmXTilesToFraction(10), rmZTilesToFraction(10), 1.0-rmXTilesToFraction(10), 1.0-rmZTilesToFraction(10), 0.01));
	rmBuildArea(centerID);

	// Build small center
	int smallCenterID=rmCreateArea("small center");
	rmSetAreaSize(smallCenterID, 0.11, 0.11);
	rmSetAreaLocation(smallCenterID, 0.5, 0.5);
	rmSetAreaCoherence(smallCenterID, 1.0);
	rmSetAreaTerrainType(smallCenterID, "SnowA");
	rmSetAreaSmoothDistance(smallCenterID, 30);
	rmAddAreaToClass(centerID, rmClassID("smallCenter"));
	rmBuildArea(smallCenterID);

	// Text
	rmSetStatusText("",0.9);

	// Constraints
	// General
	int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 7.5);
	int shortAvoidAll=rmCreateTypeDistanceConstraint("short avoid all", "all", 3.0);
	int edgeMaxConstraint=rmCreateEdgeMaxDistanceConstraint("stay close to ice", rmAreaID("center"), 12.5);
	int mediumEdgeMaxConstraint=rmCreateEdgeMaxDistanceConstraint("stay quite close to ice", rmAreaID("center"), 25.0);
	int edgeMinConstraint=rmCreateEdgeDistanceConstraint("stay away from ice", rmAreaID("center"), 5.0);
	int mediumEdgeMinConstraint=rmCreateEdgeDistanceConstraint("stay a bit away from ice", rmAreaID("center"), 17.5);
	int farEdgeMinConstraint=rmCreateEdgeDistanceConstraint("stay far away from ice", rmAreaID("center"), 50.0);
	int edgeConstraint=rmCreateAreaConstraint("stay on main area", rmAreaID("center"));
	int stayOnCenter=rmCreateAreaConstraint("stay on center", rmAreaID("small center"));
	int avoidCenter=rmCreateAreaDistanceConstraint("avoid small center", rmAreaID("small center"), 5.0);
	int centerEdgeConstraint=rmCreateEdgeMaxDistanceConstraint("stay quite close to center", rmAreaID("small center"), 15.0);

	// Settlement
	int shortAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by short distance", "AbstractSettlement", 10.0);
	int avoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by medium distance", "AbstractSettlement", 20.0);
	int farAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by great distance", "AbstractSettlement", 45.0);
	int farStartingSettleConstraint=rmCreateClassDistanceConstraint("objects avoid player TCs", rmClassID("starting settlement"), 50.0);

	// Gold
	int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 15.0);
	int farAvoidGold=rmCreateTypeDistanceConstraint("far avoid gold", "gold", 40.0);
	int mediumAvoidGold=rmCreateTypeDistanceConstraint("medium avoid gold", "gold", 30.0);
	int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 10.0);

	// Food
	int avoidHunt=rmCreateTypeDistanceConstraint("avoid huntable", "huntable", 40.0);
	int mediumAvoidHunt=rmCreateTypeDistanceConstraint("medium avoid huntable", "huntable", 22.5);
	int avoidFood=rmCreateTypeDistanceConstraint("avoid other food sources", "food", 15.0);
	int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 60.0);
	int goatConstraint=rmCreateAreaDistanceConstraint("goats avoid main", rmAreaID("center"), 5.0);
	int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 55.0);
	int goatEdgeConstraint=rmCreateBoxConstraint("goats avoid edges", rmXTilesToFraction(6), rmZTilesToFraction(6), 1.0-rmXTilesToFraction(6), 1.0-rmZTilesToFraction(6), 0.01);

	// Forest
	int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 25);
	int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 16.0);

	// Buildings
	int avoidTower=rmCreateTypeDistanceConstraint("towers avoid towers", "tower", 25.0);
	int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 15.0);

	// Text
	rmSetStatusText("",0.8);

	// Define objects
	// Starting TC
	int startingSettlementID=rmCreateObjectDef("starting settlement");
	rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
	rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
	rmSetObjectDefMinDistance(startingSettlementID, 0.0);
	rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

	// Towers
	int startingTowerID=rmCreateObjectDef("starting tower");
	rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
	rmSetObjectDefMinDistance(startingTowerID, 22.0);
	rmSetObjectDefMaxDistance(startingTowerID, 26.0);
	rmAddObjectDefConstraint(startingTowerID, avoidTower);

	// Gold
	int startingGoldID=rmCreateObjectDef("starting gold");
	rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
	rmSetObjectDefMinDistance(startingGoldID, 20.0);
	rmSetObjectDefMaxDistance(startingGoldID, 25.0);
	rmAddObjectDefConstraint(startingGoldID, avoidAll);
	rmAddObjectDefConstraint(startingGoldID, avoidGold);
	rmAddObjectDefConstraint(startingGoldID, mediumEdgeMaxConstraint);
	rmAddObjectDefConstraint(startingGoldID, mediumEdgeMinConstraint);
	rmAddObjectDefConstraint(startingGoldID, edgeMinConstraint);

	// Starting Hunt
	int closeHuntID=rmCreateObjectDef("starting hunt");
	rmAddObjectDefItem(closeHuntID, "caribou", rmRandInt(4,5), 3.0);
	rmSetObjectDefMinDistance(closeHuntID, 20.0);
	rmSetObjectDefMaxDistance(closeHuntID, 25.0);
	rmAddObjectDefConstraint(closeHuntID, avoidAll);
	rmAddObjectDefConstraint(closeHuntID, edgeMinConstraint);
	rmAddObjectDefConstraint(closeHuntID, edgeConstraint);

	// Goats
	int closeGoatsID=rmCreateObjectDef("close goats");
	rmAddObjectDefItem(closeGoatsID, "goat", 2, 2.0);
	rmSetObjectDefMinDistance(closeGoatsID, 25.0);
	rmSetObjectDefMaxDistance(closeGoatsID, 30.0);
	rmAddObjectDefConstraint(closeGoatsID, shortAvoidAll);

	// Berries
	int closeChickensID=rmCreateObjectDef("close Chickens");
	rmAddObjectDefItem(closeChickensID, "berry bush", rmRandInt(7,8), 4.0);
	rmSetObjectDefMinDistance(closeChickensID, 20.0);
	rmSetObjectDefMaxDistance(closeChickensID, 25.0);
	rmAddObjectDefConstraint(closeChickensID, avoidAll);
	rmAddObjectDefConstraint(closeChickensID, edgeMinConstraint);
	rmAddObjectDefConstraint(closeChickensID, edgeConstraint);

	// Straggler trees
	int stragglerTreeID=rmCreateObjectDef("straggler tree");
	rmAddObjectDefItem(stragglerTreeID, "pine snow", 1, 0.0);
	rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
	rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidGold);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidAll);

	// Medium object
	// Berries
	int mediumBerriesID=rmCreateObjectDef("medium berries");
	rmAddObjectDefItem(mediumBerriesID, "berry bush", rmRandInt(7,8), 4.0);
	rmSetObjectDefMinDistance(mediumBerriesID, 60.0);
	rmSetObjectDefMaxDistance(mediumBerriesID, 90.0);
	rmAddObjectDefConstraint(mediumBerriesID, avoidAll);
	rmAddObjectDefConstraint(mediumBerriesID, avoidFood);
	rmAddObjectDefConstraint(mediumBerriesID, edgeConstraint);
	rmAddObjectDefConstraint(mediumBerriesID, edgeMinConstraint);
	rmAddObjectDefConstraint(mediumBerriesID, avoidCenter);

	// Predators
	int mediumPredatorID=rmCreateObjectDef("medium predator");
	rmAddObjectDefItem(mediumPredatorID, "polar bear", rmRandInt(1,2), 4.0);
	rmSetObjectDefMinDistance(mediumPredatorID, 65.0);
	rmSetObjectDefMaxDistance(mediumPredatorID, rmXFractionToMeters(0.25));
	rmAddObjectDefConstraint(mediumPredatorID, shortAvoidAll);
	rmAddObjectDefConstraint(mediumPredatorID, avoidPredator);
	rmAddObjectDefConstraint(mediumPredatorID, edgeConstraint);

	// Hunt
	int mediumHuntID=rmCreateObjectDef("far hunt");
	if(rmRandFloat(0,1)<0.5)
		rmAddObjectDefItem(mediumHuntID, "caribou", rmRandInt(6,7), 3.0);
	else
		rmAddObjectDefItem(mediumHuntID, "elk", rmRandInt(6,7), 3.0);
	rmSetObjectDefMinDistance(mediumHuntID, 50.0);
	rmSetObjectDefMaxDistance(mediumHuntID, 70.0);
	rmAddObjectDefConstraint(mediumHuntID, shortAvoidGold);
	rmAddObjectDefConstraint(mediumHuntID, edgeMinConstraint);
	rmAddObjectDefConstraint(mediumHuntID, edgeConstraint);
	rmAddObjectDefConstraint(mediumHuntID, avoidCenter);

	// Far objects
	// Center hunt
	int centerHuntID=rmCreateObjectDef("center hunt");
	rmAddObjectDefItem(centerHuntID, "walrus", rmRandInt(4,5), 3.0);
	rmSetObjectDefMinDistance(centerHuntID, 60.0);
	rmSetObjectDefMaxDistance(centerHuntID, rmXFractionToMeters(0.225));
	rmAddObjectDefConstraint(centerHuntID, stayOnCenter);
	rmAddObjectDefConstraint(centerHuntID, mediumAvoidHunt);
	rmAddObjectDefConstraint(centerHuntID, centerEdgeConstraint);

	// Bonus hunt
	int bonusHuntID=rmCreateObjectDef("bonus hunt");
	rmAddObjectDefItem(bonusHuntID, "walrus", rmRandInt(4,5), 3.0);
	rmSetObjectDefMinDistance(bonusHuntID, 15.0);
	rmSetObjectDefMaxDistance(bonusHuntID, 30.0);
	rmAddObjectDefConstraint(bonusHuntID, stayOnCenter);
	rmAddObjectDefConstraint(bonusHuntID, shortAvoidAll);
	rmAddObjectDefConstraint(bonusHuntID, avoidFood);

	// Goats
	int goatID=rmCreateObjectDef("goats");
	rmAddObjectDefItem(goatID, "goat", 2, 2.0);
	rmSetObjectDefMinDistance(goatID, 50.0);
	rmSetObjectDefMaxDistance(goatID, rmXFractionToMeters(0.6));
	rmAddObjectDefConstraint(goatID, avoidAll);
	rmAddObjectDefConstraint(goatID, goatConstraint);
	rmAddObjectDefConstraint(goatID, avoidHerdable);
	rmAddObjectDefConstraint(goatID, goatEdgeConstraint);

	// Predators
	int farPredatorID=rmCreateObjectDef("far predator");
	rmAddObjectDefItem(farPredatorID, "polar bear", rmRandInt(1,2), 4.0);
	rmSetObjectDefMinDistance(farPredatorID, 75.0);
	rmSetObjectDefMaxDistance(farPredatorID, rmXFractionToMeters(0.35));
	rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(farPredatorID, shortAvoidAll);
	rmAddObjectDefConstraint(farPredatorID, avoidPredator);
	rmAddObjectDefConstraint(farPredatorID, edgeConstraint);

	// Random Trees
	int randomTreeID=rmCreateObjectDef("random tree");
	rmAddObjectDefItem(randomTreeID, "pine snow", 1, 0.0);
	rmSetObjectDefMinDistance(randomTreeID, 0.0);
	rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
	rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);
	rmAddObjectDefConstraint(randomTreeID, avoidAll);
	rmAddObjectDefConstraint(randomTreeID, edgeConstraint);

	// Relics
	int relicID=rmCreateObjectDef("relics");
	rmAddObjectDefItem(relicID, "relic", 1, 0.0);
	rmSetObjectDefMinDistance(relicID, 70.0);
	rmSetObjectDefMaxDistance(relicID, 90.0);
	rmAddObjectDefConstraint(relicID, edgeConstraint);
	rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 50.0));
	rmAddObjectDefConstraint(relicID, avoidAll);

	// Birds
	int farhawkID=rmCreateObjectDef("far hawks");
	rmAddObjectDefItem(farhawkID, "vulture", 1, 0.0);
	rmSetObjectDefMinDistance(farhawkID, 0.0);
	rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

	// Text
	rmSetStatusText("",0.7);

	// Place Players
	int startPos = rmRandInt(1,4);
	if(cNumberNonGaiaPlayers == 2)
	{
		if(startPos == 1)
		{
			rmPlacePlayer(1, rmRandFloat(0.23,0.24), rmRandFloat(0.23,0.24));
			rmPlacePlayer(2, rmRandFloat(0.76,0.77), rmRandFloat(0.76,0.77));
		}
	  else if(startPos == 2)
		{
			rmPlacePlayer(1, rmRandFloat(0.73,0.74), rmRandFloat(0.73,0.74));
			rmPlacePlayer(2, rmRandFloat(0.26,0.27), rmRandFloat(0.26,0.27));
		}
	  else if(startPos == 3)
		{
			rmPlacePlayer(1, rmRandFloat(0.73,0.74), rmRandFloat(0.26,0.27));
			rmPlacePlayer(2, rmRandFloat(0.26,0.27), rmRandFloat(0.73,0.74));
		}
	  else if(startPos == 4)
		{
			rmPlacePlayer(1, rmRandFloat(0.26,0.27), rmRandFloat(0.73,0.74));
			rmPlacePlayer(2, rmRandFloat(0.73,0.74), rmRandFloat(0.26,0.27));
		}
	}
	else if(cNumberTeams < 3)
	{
		rmSetTeamSpacingModifier(0.9);
		if(cNumberNonGaiaPlayers < 5)
			rmPlacePlayersCircular(0.35, 0.35, rmDegreesToRadians(5.0));
		else
			rmPlacePlayersCircular(0.34, 0.34, rmDegreesToRadians(5.0));
	}
	else
	{
		rmSetTeamSpacingModifier(0.9);
		rmPlacePlayersCircular(0.35, 0.35, rmDegreesToRadians(5.0));
	}


	// Text
	rmSetStatusText("",0.6);

	// Set up player areas.
	float playerFraction=rmAreaTilesToFraction(1000);
	for(i=1; <cNumberPlayers)
	{
		int id=rmCreateArea("Player"+i);
		rmSetPlayerArea(i, id);
		rmSetAreaSize(id, 1.0*playerFraction, 1.0*playerFraction);
		rmSetAreaCoherence(id, 0.85);
		rmSetAreaLocPlayer(id, i);
		rmSetAreaSmoothDistance(id, 10);
		rmSetAreaTerrainType(id, "SnowA");
	}

	rmBuildAllAreas();

	// Starting TC
	rmPlaceObjectDefPerPlayer(startingSettlementID, true);

	// Fair gold
	if(cNumberNonGaiaPlayers < 5)
	{
		id=rmAddFairLoc("first gold", false, true, 75, 90, 20, 10);
		rmAddFairLocConstraint(id, edgeMinConstraint);
		rmAddFairLocConstraint(id, edgeMaxConstraint);
		rmAddFairLocConstraint(id, edgeConstraint);
		rmAddFairLocConstraint(id, avoidGold);
	}
	if(cNumberNonGaiaPlayers == 2)
		id=rmAddFairLoc("second gold", true, false, 100, 105, 40, 10);
	else if(cNumberNonGaiaPlayers < 5)
		id=rmAddFairLoc("second gold", true, false, 60, 70, 40, 10);
	else
	{
		id=rmAddFairLoc("second gold", true, false, 60, 70, 70, 10);
		rmAddFairLocConstraint(id, farStartingSettleConstraint);
	}
	rmAddFairLocConstraint(id, edgeMinConstraint);
	rmAddFairLocConstraint(id, edgeMaxConstraint);
	rmAddFairLocConstraint(id, edgeConstraint);
	rmAddFairLocConstraint(id, avoidGold);
	if(cNumberNonGaiaPlayers == 2)
		id=rmAddFairLoc("third gold", true, false, 115, 125, 40, 10);
	else if(cNumberNonGaiaPlayers < 5)
		id=rmAddFairLoc("third gold", true, false, 100, 105, 40, 10);
	else
	{
		id=rmAddFairLoc("third gold", true, false, 100, 105, 70, 10);
		rmAddFairLocConstraint(id, farStartingSettleConstraint);
	}
	rmAddFairLocConstraint(id, edgeMinConstraint);
	rmAddFairLocConstraint(id, edgeMaxConstraint);
	rmAddFairLocConstraint(id, edgeConstraint);
	rmAddFairLocConstraint(id, farAvoidGold);

	if(rmPlaceFairLocs())
	{
		id=rmCreateObjectDef("fair gold");
		rmAddObjectDefItem(id, "Gold mine", 1, 0.0);
		for(i=1; <cNumberPlayers)
		{
			for(j=0; <rmGetNumberFairLocs(i))
			{
				int firstGoldArea = rmCreateArea("fair gold area"+i +j);
				rmSetAreaLocation(firstGoldArea, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j));
				rmBuildArea(firstGoldArea);
				rmPlaceObjectDefAtAreaLoc(id, i, firstGoldArea);
			}
		}
	}

	rmResetFairLocs();

	// Text
	rmSetStatusText("",0.5);

	// Settlement 1
	if(cNumberNonGaiaPlayers == 2)
		id=rmAddFairLoc("Settlement", false, true, 60, 65, 50, 65);
	else
		id=rmAddFairLoc("Settlement", false, true, 70, 80, 40, 40);
	rmAddFairLocConstraint(id, mediumAvoidGold);

	if(rmPlaceFairLocs())
	{
		id=rmCreateObjectDef("settlement 1");
		rmAddObjectDefItem(id, "Settlement", 1, 0.0);
		for(i=1; <cNumberPlayers)
		{
			for(j=0; <rmGetNumberFairLocs(i))
				rmPlaceObjectDefAtLoc(id, i, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j), 1);
		}
	}

	rmResetFairLocs();

	// Settlement 2
	if(cNumberNonGaiaPlayers == 2)
	{
		if(rmRandFloat(0,1)<0.85)
			id=rmAddFairLoc("Settlement", true, false, 75, 80, 50, 70);
		else
			id=rmAddFairLoc("Settlement", false, true, 65, 70, 50, 55);
			rmAddFairLocConstraint(id, farAvoidSettlement);
	}
	else if(cNumberNonGaiaPlayers < 5)
	{
		id=rmAddFairLoc("Settlement", true, true, 80, 85, 60, 10);
		rmAddFairLocConstraint(id, farAvoidSettlement);
	}
	else
	{
		if(rmRandFloat(0,1)<0.33)
			id=rmAddFairLoc("Settlement", true, false, 80, 95, 60, 10);
		else
			id=rmAddFairLoc("Settlement", true, true, 80, 90, 60, 10);
		rmAddFairLocConstraint(id, farAvoidSettlement);
		rmAddFairLocConstraint(id, farEdgeMinConstraint);
	}
	rmAddFairLocConstraint(id, avoidGold);

	if(rmPlaceFairLocs())
	{
		id=rmCreateObjectDef("bonus settlement");
		rmAddObjectDefItem(id, "Settlement", 1, 0.0);
		for(i=1; <cNumberPlayers)
		{
			for(j=0; <rmGetNumberFairLocs(i))
			{
				int bonusSettleArea = rmCreateArea("bonus settlement area"+i +j);
				rmSetAreaLocation(bonusSettleArea, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j));
				rmBuildArea(bonusSettleArea);
				rmPlaceObjectDefAtAreaLoc(id, i, bonusSettleArea);
			}
		}
	}

	// Text
	rmSetStatusText("",0.4);

	// Slight Elevation
	int numTries=8*cNumberNonGaiaPlayers;
	int failCount=0;
	for(i=0; <numTries)
	{
		int elevID=rmCreateArea("elev"+i);
		rmSetAreaSize(elevID, rmAreaTilesToFraction(15), rmAreaTilesToFraction(80));
		rmSetAreaWarnFailure(elevID, false);
		rmAddAreaConstraint(elevID, avoidBuildings);
		rmAddAreaConstraint(elevID, edgeConstraint);
		rmSetAreaBaseHeight(elevID, rmRandFloat(4.0, 7.0));
		rmSetAreaHeightBlend(elevID, 2);
		rmSetAreaMinBlobs(elevID, 1);
		rmSetAreaMaxBlobs(elevID, 5);
		rmSetAreaMinBlobDistance(elevID, 16.0);
		rmSetAreaMaxBlobDistance(elevID, 40.0);
		rmSetAreaCoherence(elevID, 0.0);

		if(rmBuildArea(elevID)==false)
		{
			// Stop trying once we fail 6 times in a row.
			failCount++;
			if(failCount==6)
				break;
		}
		else
			failCount=0;
	}

	// Texture embellishment
	for(i=1; <cNumberPlayers*15)
	{
		int id1=rmCreateArea("beautification"+i);
		rmSetAreaSize(id1, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id1, "SnowB");
		rmAddAreaConstraint(id1, edgeConstraint);
		rmSetAreaMinBlobs(id1, 1);
		rmSetAreaMaxBlobs(id1, 5);
		rmSetAreaWarnFailure(id1, false);
		rmSetAreaMinBlobDistance(id1, 16.0);
		rmSetAreaMaxBlobDistance(id1, 40.0);
		rmSetAreaCoherence(id1, 0.6);
		rmBuildArea(id1);
	}

	 for(i=1; <cNumberPlayers*10)
	{
		int id2=rmCreateArea("more beautification"+i);
		rmSetAreaSize(id2, rmAreaTilesToFraction(60), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id2, "SnowSand25");
		rmAddAreaConstraint(id2, edgeConstraint);
		rmSetAreaMinBlobs(id2, 1);
		rmSetAreaMaxBlobs(id2, 5);
		rmSetAreaWarnFailure(id2, false);
		rmSetAreaMinBlobDistance(id2, 16.0);
		rmSetAreaMaxBlobDistance(id2, 40.0);
		rmSetAreaCoherence(id2, 0.4);
		rmBuildArea(id2);
	}

	// Text
	rmSetStatusText("",0.3);

	// Place close objects
	// Starting gold
	rmPlaceObjectDefPerPlayer(startingGoldID, false, 2);

	// Towers
	rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

	// Close hunt
	rmPlaceObjectDefPerPlayer(closeHuntID, false);

	// Starting berries
	rmPlaceObjectDefPerPlayer(closeChickensID, false);

	// Medium objects
	// Hunt
	rmPlaceObjectDefPerPlayer(mediumHuntID, false);

	// Berries
	rmPlaceObjectDefPerPlayer(mediumBerriesID, false);

	// Far objects
	// Center hunt
	if(cNumberNonGaiaPlayers == 2)
		rmPlaceObjectDefPerPlayer(centerHuntID, false, rmRandInt(1,2));
	else
	{
		rmPlaceObjectDefPerPlayer(centerHuntID, false, 1);
		rmPlaceObjectDefAtLoc(bonusHuntID, 0, 0.5, 0.5, cNumberNonGaiaPlayers);
	}

	// Goats
	rmPlaceObjectDefPerPlayer(goatID, false, rmRandInt(5,7));

	// Starting goats
	// Placed after the other goats to bypass avoidHerdable
	rmPlaceObjectDefPerPlayer(closeGoatsID, true);

	// Text
	rmSetStatusText("",0.2);

	// Forest
	int forestCount=13*cNumberNonGaiaPlayers;
	failCount=0;
	for(i=0; <forestCount)
	{
		int forestID=rmCreateArea("forest"+i);
		rmSetAreaSize(forestID, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaForestType(forestID, "snow pine forest");
		rmAddAreaToClass(forestID, classForest);
		rmSetAreaMinBlobs(forestID, 1);
		rmSetAreaMaxBlobs(forestID, 3);
		rmAddAreaConstraint(forestID, edgeMinConstraint);
		rmAddAreaConstraint(forestID, edgeConstraint);
		rmAddAreaConstraint(forestID, avoidAll);
		rmAddAreaConstraint(forestID, forestSettleConstraint);
		rmAddAreaConstraint(forestID, forestConstraint);
		rmAddAreaConstraint(forestID, avoidBuildings);
		rmSetAreaMinBlobDistance(forestID, 16.0);
		rmSetAreaMaxBlobDistance(forestID, 40.0);
		rmSetAreaCoherence(forestID, 0.0);
		if(rmBuildArea(forestID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3)
				break;
		}
		else
			failCount=0;
	}

	// Other stuff
	// Straggler trees
	rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(4,7));

	// Predators
	rmPlaceObjectDefPerPlayer(mediumPredatorID, false);
	rmPlaceObjectDefPerPlayer(farPredatorID, false);

	// Relics
	rmPlaceObjectDefPerPlayer(relicID, false, 1);

	// Random trees
	rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 10*cNumberNonGaiaPlayers);

	// Hawks
	rmPlaceObjectDefPerPlayer(farhawkID, false, 2);

	// Text
	rmSetStatusText("",0.1);

	// Embellishment
	int rock1ID=rmCreateObjectDef("rock 1 group");
	rmAddObjectDefItem(rock1ID, "rock granite sprite", 1, 2.0);
	rmSetObjectDefMinDistance(rock1ID, 0.0);
	rmSetObjectDefMaxDistance(rock1ID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(rock1ID, avoidAll);
	rmAddObjectDefConstraint(rock1ID, edgeConstraint);
	rmPlaceObjectDefAtLoc(rock1ID, 0, 0.5, 0.5, 12*cNumberNonGaiaPlayers);

	int rock2ID=rmCreateObjectDef("rock 2 group");
	rmAddObjectDefItem(rock2ID, "rock granite small", 1, 2.0);
	rmSetObjectDefMinDistance(rock2ID, 0.0);
	rmSetObjectDefMaxDistance(rock2ID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(rock2ID, avoidAll);
	rmAddObjectDefConstraint(rock2ID, edgeConstraint);
	rmPlaceObjectDefAtLoc(rock2ID, 0, 0.5, 0.5, 7*cNumberNonGaiaPlayers);

	int runeID=rmCreateObjectDef("runestone");
	rmAddObjectDefItem(runeID, "runestone", 1, 0.0);
	rmSetObjectDefMinDistance(runeID, 0.0);
	rmSetObjectDefMaxDistance(runeID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(runeID, avoidAll);
	rmAddObjectDefConstraint(runeID, edgeConstraint);
	rmAddObjectDefConstraint(runeID, avoidBuildings);
	rmPlaceObjectDefAtLoc(runeID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

	int driftID=rmCreateObjectDef("snow drift");
	rmAddObjectDefItem(driftID, "snow drift", 1, 0.0);
	rmSetObjectDefMinDistance(driftID, 0.0);
	rmSetObjectDefMaxDistance(driftID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(driftID, avoidAll);
	rmAddObjectDefConstraint(driftID, edgeConstraint);
	rmAddObjectDefConstraint(driftID, rmCreateTypeDistanceConstraint("drift vs drift", "snow drift", 25.0));
	rmPlaceObjectDefAtLoc(driftID, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);

	// Text
	rmSetStatusText("", 0.01);

	// Notify players
	if(cNumberTeams > 2)
	{
		rmCreateTrigger("team notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
	else if(cNumberNonGaiaPlayers > 6)
	{
		rmCreateTrigger("player notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
}