// Nile Delta 1.0
// Script: RR
// Concept: RR/Bubble

// Main entry point for random map script
void main(void)
{
	// Text
	rmSetStatusText("",1.0);

	// Set size
	int playerTiles=9000;
	int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
	int skew=rmRandInt(1,2);
	if(cNumberNonGaiaPlayers < 5)
	{
		rmSetMapSize(size, size);
		rmEchoInfo("Map size="+size+"m x "+size+"m");
	}
	else
	{
		skew=1;
		rmSetMapSize(size, size*1.1);
		rmEchoInfo("Map size="+size+"m x "+size*1.1+"m");
	}

	// Set up default water
	rmSetSeaLevel(1.0);
	rmSetSeaType("Egyptian Nile Shallow");

	// Init map
	rmTerrainInitialize("water");
	// rmSetLightingSet("Default");

	// Define Classes
	int classLand=rmDefineClass("land");
	int classOcean=rmDefineClass("ocean");
	int classPlayer=rmDefineClass("player");
	int classBonusHuntable=rmDefineClass("bonus huntable");
	int classForest=rmDefineClass("forest");
	rmDefineClass("starting settlement");
	rmDefineClass("center line");

	// Constraints
	// General
	int divisionConstraint=0;
	if(skew==1)
		divisionConstraint=rmCreateBoxConstraint("divide map", 0.45, 0.0, 0.55, 1.0, 0.01);
	else
		divisionConstraint=rmCreateBoxConstraint("divide map", 0.0, 0.45, 1.0, 0.55, 0.01);
	int edgeConstraint=rmCreateBoxConstraint("avoid edges", rmXTilesToFraction(4), rmZTilesToFraction(4), 1.0-rmXTilesToFraction(4), 1.0-rmZTilesToFraction(4), 0.01);
	int mediumEdgeConstraint=rmCreateBoxConstraint("medium avoid edges", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8), 0.01);
	int longEdgeConstraint=rmCreateBoxConstraint("long avoid edges", rmXTilesToFraction(12), rmZTilesToFraction(12), 1.0-rmXTilesToFraction(12), 1.0-rmZTilesToFraction(12), 0.01);
	int shortAvoidAll=rmCreateTypeDistanceConstraint("short avoid all", "all", 3.0);
	int mediumAvoidAll=rmCreateTypeDistanceConstraint("medium avoid all", "all", 8.0);
	int tinyCenterLineConstraint=rmCreateClassDistanceConstraint("stay a tiny bit away from center line", rmClassID("center line"), 1.0);
	int shortCenterLineConstraint=rmCreateClassDistanceConstraint("stay a bit away from center line", rmClassID("center line"), 15.0);
	int mediumCenterLineConstraint=rmCreateClassDistanceConstraint("stay away from center line", rmClassID("center line"), 20.0);
	int farCenterLineConstraint=rmCreateClassDistanceConstraint("stay far away from center line", rmClassID("center line"), 50.0);

	// Terrain
	int oceanConstraint=0;
	int smallOceanConstraint=0;
	if(skew==1)
	{
		if(cNumberNonGaiaPlayers < 3)
			oceanConstraint=rmCreateBoxConstraint("limit ocean", 0.0, 0.48, 1.0, 1.0, 0.01);
		else if(cNumberNonGaiaPlayers < 5)
		{
			oceanConstraint=rmCreateBoxConstraint("limit ocean", 0.0, 0.54, 1.0, 1.0, 0.01);
			smallOceanConstraint=rmCreateBoxConstraint("limit small ocean", 0.0, 0.0, 1.0, 0.22, 0.01);
		}
		else
		{
			oceanConstraint=rmCreateBoxConstraint("limit ocean", 0.0, 0.54, 1.0, 1.0, 0.01);
			smallOceanConstraint=rmCreateBoxConstraint("limit small ocean", 0.0, 0.0, 1.0, 0.18, 0.01);
		}
	}
	else
	{
		if(cNumberNonGaiaPlayers < 3)
			oceanConstraint=rmCreateBoxConstraint("limit ocean", 0.48, 0.0, 1.0, 1.0, 0.01);
		else if(cNumberNonGaiaPlayers < 5)
		{
			oceanConstraint=rmCreateBoxConstraint("limit ocean", 0.54, 0.0, 1.0, 1.0, 0.01);
			smallOceanConstraint=rmCreateBoxConstraint("limit small ocean", 0.0, 0.0, 0.22, 1.0, 0.01);
		}
		else
		{
			oceanConstraint=rmCreateBoxConstraint("limit ocean", 0.54, 0.0, 1.0, 1.0, 0.01);
			smallOceanConstraint=rmCreateBoxConstraint("limit small ocean", 0.0, 0.0, 0.18, 1.0, 0.01);
		}
	}
	int oceanAvoidsLand=rmCreateClassDistanceConstraint("oceans avoid land", classLand, 1.0);
	int farAvoidOcean=rmCreateClassDistanceConstraint("avoid ocean by far", classOcean, 25.0);
	int mediumAvoidOcean=rmCreateClassDistanceConstraint("avoid ocean a bit", classOcean, 15.0);
	int shortAvoidOcean=rmCreateClassDistanceConstraint("avoid ocean by a tiny bit", classOcean, 7.5);
	int nearShore=rmCreateTerrainMaxDistanceConstraint("near shore", "water", true, 8.0);

	// Shallows
	int shallowConstraintSW=0;
	int shallowConstraintNE=0;
	if(skew==1)
	{
		shallowConstraintSW=rmCreateBoxConstraint("SW avoids shallows", 0.0, 0.0, 0.47, 1.0, 0.01);
		shallowConstraintNE=rmCreateBoxConstraint("NE avoids shallows", 0.53, 0.0, 1.0, 1.0, 0.01);
	}
	else
	{
		shallowConstraintSW=rmCreateBoxConstraint("SE avoids shallows", 0.0, 0.0, 1.0, 0.47, 0.01);
		shallowConstraintNE=rmCreateBoxConstraint("NW avoids shallows", 0.0, 0.53, 1.0, 1.0, 0.01);
	}

	// Settlements
	int startingSettleConstraint=rmCreateClassDistanceConstraint("avoid starting settlement", rmClassID("starting settlement"), 30.0);
	int avoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC", "AbstractSettlement", 10.0);
	int settlementVsSettlement=rmCreateTypeDistanceConstraint("TCs avoid TCs", "AbstractSettlement", 25.0);
	int settlementAvoidsOcean=rmCreateClassDistanceConstraint("settlements avoid ocean by far", classOcean, 35.0);

	// Gold
	int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 8.0);
	int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
	int farAvoidGold=rmCreateTypeDistanceConstraint("far avoid gold", "gold", 40.0);

	// Food
	int foodZone=0;
	if(skew==1)
		foodZone=rmCreateBoxConstraint("food zone", 0.0, 0.15, 1.0, 0.15, 0.01);
	else
		foodZone=rmCreateBoxConstraint("food zone", 0.15, 0.0, 0.15, 1.0, 0.01);
	int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 40.0);
	int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 40.0);
	int mediumAvoidHunt=rmCreateTypeDistanceConstraint("avoid huntable by a bit", "huntable", 30.0);
	int shortAvoidHunt=rmCreateTypeDistanceConstraint("short avoid huntable", "huntable", 20.0);
	int avoidBonusHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", rmClassID("bonus huntable"),35.0);
	int shortFishVsFishID=rmCreateTypeDistanceConstraint("short fish v fish", "fish", 10.0);
	int longFishVsFishID=rmCreateTypeDistanceConstraint("long fish v fish", "fish", 15.0);
	int extraFishVsFishID=rmCreateTypeDistanceConstraint("extra fish v fish", "fish", 30.0);
	int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 20.0);
	int fishLand=rmCreateTerrainDistanceConstraint("fish land", "land", true, 6.0);
	int fishNearShore=rmCreateTerrainMaxDistanceConstraint("fish near shore", "water", true, 25.0);

	// Wood
	int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 22.0);
	int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);

	// Buildings
	int avoidTower=rmCreateTypeDistanceConstraint("avoid tower", "tower", 25.0);
	int shortAvoidTower=rmCreateTypeDistanceConstraint("short avoid tower", "tower", 15.0);

	// Text
	rmSetStatusText("",0.9);

	// Define objects
	// Starting Settlement
	int startingSettlementID=rmCreateObjectDef("starting settlement");
	rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
	rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
	rmSetObjectDefMinDistance(startingSettlementID, 0.0);
	rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

	// Towers
	int startingTowerID=rmCreateObjectDef("starting tower");
	rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
	rmSetObjectDefMinDistance(startingTowerID, 22.0);
	rmSetObjectDefMaxDistance(startingTowerID, 24.0);
	rmAddObjectDefConstraint(startingTowerID, avoidTower);
	rmAddObjectDefConstraint(startingTowerID, farAvoidOcean);

	// Gold
	int startingGoldID=rmCreateObjectDef("starting gold");
	rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
	rmSetObjectDefMinDistance(startingGoldID, 20.0);
	rmSetObjectDefMaxDistance(startingGoldID, 25.0);
	rmAddObjectDefConstraint(startingGoldID, mediumAvoidOcean);
	rmAddObjectDefConstraint(startingGoldID, shortAvoidAll);

	// Berries
	int startingFoodID=rmCreateObjectDef("close food");
	rmAddObjectDefItem(startingFoodID, "berry bush", rmRandInt(5,7), 4.0);
	rmSetObjectDefMinDistance(startingFoodID, 20.0);
	rmSetObjectDefMaxDistance(startingFoodID, 25.0);
	rmAddObjectDefConstraint(startingFoodID, shortAvoidAll);

	// Pigs
	int startingPigsID=rmCreateObjectDef("close pigs");
	rmAddObjectDefItem(startingPigsID, "pig", 4, 2.0);
	rmSetObjectDefMinDistance(startingPigsID, 25.0);
	rmSetObjectDefMaxDistance(startingPigsID, 30.0);
	rmAddObjectDefConstraint(startingPigsID, shortAvoidAll);
	rmAddObjectDefConstraint(startingPigsID, edgeConstraint);

	// Straggler trees
	int stragglerTreeID=rmCreateObjectDef("straggler tree");
	rmAddObjectDefItem(stragglerTreeID, "palm", 1, 0.0);
	rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
	rmSetObjectDefMaxDistance(stragglerTreeID, 20.0);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidGold);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidAll);

	// Medium objects
	// Medium gold
	int mediumGoldID=rmCreateObjectDef("medium gold");
	rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(mediumGoldID, 40.0);
	rmSetObjectDefMaxDistance(mediumGoldID, 60.0);
	rmAddObjectDefConstraint(mediumGoldID, avoidGold);
	rmAddObjectDefConstraint(mediumGoldID, edgeConstraint);
	rmAddObjectDefConstraint(mediumGoldID, startingSettleConstraint);
	rmAddObjectDefConstraint(mediumGoldID, farAvoidOcean);
	rmAddObjectDefConstraint(mediumGoldID, farCenterLineConstraint);

	// Water hippos
	int waterHippoID=rmCreateObjectDef("water hippos");
	rmAddObjectDefItem(waterHippoID, "hippo", rmRandInt(3,4), 2.0);
	rmSetObjectDefMinDistance(waterHippoID, 15.0);
	rmSetObjectDefMaxDistance(waterHippoID, 80.0);
	rmAddObjectDefConstraint(waterHippoID, shortAvoidTower);
	rmAddObjectDefConstraint(waterHippoID, nearShore);
	rmAddObjectDefConstraint(waterHippoID, mediumCenterLineConstraint);
	rmAddObjectDefConstraint(waterHippoID, mediumEdgeConstraint);

	// Medium hunt
	int mediumHuntID=rmCreateObjectDef("medium hunt");
	rmAddObjectDefItem(mediumHuntID, "gazelle", rmRandInt(3,4), 3.0);
	rmSetObjectDefMinDistance(mediumHuntID, 55.0);
	rmSetObjectDefMaxDistance(mediumHuntID, 60.0);
	rmAddObjectDefConstraint(mediumHuntID, avoidTower);
	rmAddObjectDefConstraint(mediumHuntID, mediumAvoidHunt);
	rmAddObjectDefConstraint(mediumHuntID, shortAvoidAll);
	rmAddObjectDefConstraint(mediumHuntID, edgeConstraint);
	rmAddObjectDefConstraint(mediumHuntID, mediumAvoidOcean);
	rmAddObjectDefConstraint(mediumHuntID, farCenterLineConstraint);

	// Medium pigs
	int mediumPigsID=rmCreateObjectDef("medium pigs");
	rmAddObjectDefItem(mediumPigsID, "pig", 2, 2.0);
	rmSetObjectDefMinDistance(mediumPigsID, 55.0);
	rmSetObjectDefMaxDistance(mediumPigsID, 65.0);
	rmAddObjectDefConstraint(mediumPigsID, avoidHerdable);
	rmAddObjectDefConstraint(mediumPigsID, shortAvoidAll);
	rmAddObjectDefConstraint(mediumPigsID, edgeConstraint);
	rmAddObjectDefConstraint(mediumPigsID, shortAvoidOcean);
	rmAddObjectDefConstraint(mediumPigsID, mediumCenterLineConstraint);

	// Medium predator
	int predatorID=rmCreateObjectDef("predator");
	rmAddObjectDefItem(predatorID, "lion", 2, 4.0);
	rmSetObjectDefMinDistance(predatorID, 60.0);
	rmSetObjectDefMaxDistance(predatorID, 70.0);
	rmAddObjectDefConstraint(predatorID, shortAvoidAll);
	rmAddObjectDefConstraint(predatorID, edgeConstraint);
	rmAddObjectDefConstraint(predatorID, farAvoidOcean);
	rmAddObjectDefConstraint(predatorID, avoidPredator);
	rmAddObjectDefConstraint(predatorID, startingSettleConstraint);

	// Far objects
	// Far gold
	int farGoldID=rmCreateObjectDef("far gold");
	rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(farGoldID, 60.0);
	rmSetObjectDefMaxDistance(farGoldID, 100.0);
	rmAddObjectDefConstraint(farGoldID, farAvoidGold);
	rmAddObjectDefConstraint(farGoldID, edgeConstraint);
	rmAddObjectDefConstraint(farGoldID, avoidSettlement);
	rmAddObjectDefConstraint(farGoldID, startingSettleConstraint);
	rmAddObjectDefConstraint(farGoldID, farAvoidOcean);
	rmAddObjectDefConstraint(farGoldID, shortCenterLineConstraint);

	// Random hunt
	int randomHuntID=rmCreateObjectDef("random hunt");
	rmAddObjectDefToClass(randomHuntID, classBonusHuntable);
	if(rmRandFloat(0,1)>0.66)
	{
		rmAddObjectDefItem(randomHuntID, "gazelle", rmRandInt(4,6), 4.0);
		rmAddObjectDefItem(randomHuntID, "giraffe", rmRandInt(3,4), 4.0);
	}
	else if(rmRandFloat(0,1)>0.33)
	{
		rmAddObjectDefItem(randomHuntID, "gazelle", rmRandInt(3,4), 4.0);
		rmAddObjectDefItem(randomHuntID, "giraffe", rmRandInt(2,3), 4.0);
	}
	else
		rmAddObjectDefItem(randomHuntID, "giraffe", 2, 4.0);
	rmSetObjectDefMinDistance(randomHuntID, 55.0);
	rmSetObjectDefMaxDistance(randomHuntID, 75.0);
	rmAddObjectDefConstraint(randomHuntID, avoidTower);
	rmAddObjectDefConstraint(randomHuntID, shortAvoidHunt);
	rmAddObjectDefConstraint(randomHuntID, shortAvoidAll);
	rmAddObjectDefConstraint(randomHuntID, edgeConstraint);
	rmAddObjectDefConstraint(randomHuntID, farAvoidOcean);
	rmAddObjectDefConstraint(randomHuntID, avoidBonusHuntable);
	rmAddObjectDefConstraint(randomHuntID, farCenterLineConstraint);

	// Far buffalos
	int buffaloID=rmCreateObjectDef("buffalos");
	rmAddObjectDefItem(buffaloID, "water buffalo", rmRandInt(3,4), 2.0);
	rmSetObjectDefMinDistance(buffaloID, 60.0);
	rmSetObjectDefMaxDistance(buffaloID, 135.0);
	rmAddObjectDefConstraint(buffaloID, mediumCenterLineConstraint);
	rmAddObjectDefConstraint(buffaloID, mediumAvoidHunt);
	rmAddObjectDefConstraint(buffaloID, avoidBonusHuntable);
	rmAddObjectDefConstraint(buffaloID, edgeConstraint);
	rmAddObjectDefConstraint(buffaloID, shortAvoidAll);
	rmAddObjectDefConstraint(buffaloID, foodZone);
	rmAddObjectDefToClass(buffaloID, classBonusHuntable);

	// Far pigs
	int farPigsID=rmCreateObjectDef("far pigs");
	rmAddObjectDefItem(farPigsID, "pig", 2, 2.0);
	rmSetObjectDefMinDistance(farPigsID, 65.0);
	rmSetObjectDefMaxDistance(farPigsID, 95.0);
	rmAddObjectDefConstraint(farPigsID, avoidHerdable);
	rmAddObjectDefConstraint(farPigsID, shortAvoidAll);
	rmAddObjectDefConstraint(farPigsID, edgeConstraint);
	rmAddObjectDefConstraint(farPigsID, mediumAvoidOcean);

	// Random Trees
	int randomTreeID=rmCreateObjectDef("random tree");
	rmAddObjectDefItem(randomTreeID, "palm", 1, 0.0);
	rmSetObjectDefMinDistance(randomTreeID, 0.0);
	rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
	rmAddObjectDefConstraint(randomTreeID, shortAvoidAll);
	rmAddObjectDefConstraint(randomTreeID, rmCreateClassDistanceConstraint("short avoid starting settlement", rmClassID("starting settlement"), 15.0));

	// Relics
	int relicID=rmCreateObjectDef("relics");
	rmAddObjectDefItem(relicID, "relic", 1, 0.0);
	rmSetObjectDefMinDistance(relicID, 70.0);
	rmSetObjectDefMaxDistance(relicID, rmXFractionToMeters(0.3));
	rmAddObjectDefConstraint(relicID, mediumCenterLineConstraint);
	rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 50.0));
	rmAddObjectDefConstraint(relicID, shortAvoidAll);
	rmAddObjectDefConstraint(relicID, mediumAvoidOcean);

	// Birds
	int farhawkID=rmCreateObjectDef("far hawks");
	rmAddObjectDefItem(farhawkID, "vulture", 1, 0.0);
	rmSetObjectDefMinDistance(farhawkID, 0.0);
	rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

	// Text
	rmSetStatusText("",0.8);

	// Generic stuff
	// Player placement
	if(cNumberTeams < 3)
	{
		if(skew==1)
		{
			if(cNumberNonGaiaPlayers == 2)
				rmPlacePlayersLine(0.18, 0.58, 0.82, 0.58);
			else if(cNumberNonGaiaPlayers < 5)
			{
				rmSetPlacementTeam(0);
				rmPlacePlayersLine(0.225, 0.17, 0.225, 0.65, 0.01);
				rmSetPlacementTeam(1);
				rmPlacePlayersLine(0.775, 0.17, 0.775, 0.65, 0.01);
			}
			else if(cNumberNonGaiaPlayers < 7)
			{
				rmSetPlacementTeam(0);
				rmPlacePlayersLine(0.24, 0.15, 0.19, 0.77, 0.01);
				rmSetPlacementTeam(1);
				rmPlacePlayersLine(0.76, 0.15, 0.81, 0.77, 0.01);
			}
			else
			{
				rmSetPlacementTeam(0);
				rmPlacePlayersLine(0.25, 0.12, 0.18, 0.87, 0.01);
				rmSetPlacementTeam(1);
				rmPlacePlayersLine(0.75, 0.12, 0.82, 0.87, 0.01);
			}
		}
		else
		{
			if(cNumberNonGaiaPlayers == 2)
				rmPlacePlayersLine(0.59, 0.18, 0.59, 0.82);
			else
			{
				rmSetPlacementTeam(0);
				rmPlacePlayersLine(0.17, 0.225, 0.65, 0.225, 0.01);
				rmSetPlacementTeam(1);
				rmPlacePlayersLine(0.17, 0.775, 0.65, 0.775, 0.01);
			}
		}
	}
	else
		rmPlacePlayersLine(0.08, 0.45, 0.92, 0.45);
		// Impossible to get a good placement with 3+ teams


	// Divide map
	int divisionID=rmCreateArea("division");
	rmSetAreaSize(divisionID, 1, 1);
	rmSetAreaLocation(divisionID, 0.5, 0.5);
	rmAddAreaToClass(divisionID, rmClassID("center line"));
	rmSetAreaWarnFailure(divisionID, false);
	rmAddAreaConstraint(divisionID, divisionConstraint);
	rmBuildArea(divisionID);

	// Define peninsula constants
	int size1=0;
	int size2=0;
	int size3=0;
	if(cNumberNonGaiaPlayers == 2)
	{
		size1=0.64*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
		size2=0.15*0.64*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
		size3=0.46*0.64*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
	}
	else if(cNumberNonGaiaPlayers < 5)
	{
		size1=0.7*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
		size2=0.15*0.7*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
		size3=0.35*0.7*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
	}
	else
	{
		size1=0.7*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
		size2=0.15*0.7*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
		size3=0.28*0.7*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
	}

	float loc1x=0.0;
	float loc1z=0.0;
	float loc2x=0.0;
	float loc2z=0.0;
	float loc3x=0.0;
	float loc3z=0.0;
	float loc4x=0.0;
	float loc4z=0.0;
	float loc5x=0.0;
	float loc5z=0.0;
	float loc6x=0.0;
	float loc6z=0.0;

	if(skew==1)
	{
		loc1x=0.0;
		loc1z=0.4;
		loc2x=0.3;
		loc2z=0.65;
		loc3x=0.0;
		loc3z=0.0;
		loc4x=1.0;
		loc4z=0.4;
		loc5x=0.7;
		loc5z=0.65;
		loc6x=1.0;
		loc6z=0.0;
	}
	else
	{
		loc1x=0.4;
		loc1z=0.0;
		loc2x=0.65;
		loc2z=0.7;
		loc3x=0.0;
		loc3z=0.0;
		loc4x=0.4;
		loc4z=1.0;
		loc5x=0.65;
		loc5z=0.7;
		loc6x=0.0;
		loc6z=1.0;
	}

	// Create southwestern peninsula
	// Part 1
	int island1ID=rmCreateArea("peninsula 1");
	rmSetAreaSize(island1ID, rmXMetersToFraction(size1), rmZMetersToFraction(size1));
	rmSetAreaLocation(island1ID, loc1x, loc1z);
	rmAddAreaToClass(island1ID, classLand);
	rmSetAreaWarnFailure(island1ID, false);
	rmSetAreaMinBlobs(island1ID, 1);
	rmSetAreaMaxBlobs(island1ID, 5);
	rmSetAreaMinBlobDistance(island1ID, 5.0);
	rmSetAreaMaxBlobDistance(island1ID, 10.0);
	rmSetAreaCoherence(island1ID, 0.6);
	rmSetAreaBaseHeight(island1ID, 2.0);
	rmSetAreaSmoothDistance(island1ID, 10.0);
	rmSetAreaHeightBlend(island1ID, 2.0);
	rmSetAreaTerrainType(island1ID, "SandA");
	rmAddAreaConstraint(island1ID, shallowConstraintSW);

	// Part 2
	int island2ID=rmCreateArea("peninsula 2");
	rmSetAreaSize(island2ID,rmXMetersToFraction(size2), rmZMetersToFraction(size2));
	rmSetAreaLocation(island2ID, loc2x, loc2z);
	rmAddAreaToClass(island2ID, classLand);
	rmSetAreaWarnFailure(island2ID, false);
	rmSetAreaMinBlobs(island2ID, 1);
	rmSetAreaMaxBlobs(island2ID, 5);
	rmSetAreaMinBlobDistance(island2ID, 5.0);
	rmSetAreaMaxBlobDistance(island2ID, 10.0);
	rmSetAreaCoherence(island2ID, 0.5);
	rmSetAreaBaseHeight(island2ID, 2.0);
	rmSetAreaSmoothDistance(island2ID, 10.0);
	rmSetAreaHeightBlend(island2ID, 2.0);
	rmSetAreaTerrainType(island2ID, "SandA");
	rmAddAreaConstraint(island2ID, shallowConstraintSW);

	// Part 3
	int island3ID=rmCreateArea("peninsula 3");
	rmSetAreaSize(island3ID, rmXMetersToFraction(size3), rmZMetersToFraction(size3));
	rmSetAreaLocation(island3ID, loc3x, loc3z);
	rmAddAreaToClass(island3ID, classLand);
	rmSetAreaWarnFailure(island3ID, false);
	rmSetAreaMinBlobs(island3ID, 1);
	rmSetAreaMaxBlobs(island3ID, 5);
	rmSetAreaMinBlobDistance(island3ID, 5.0);
	rmSetAreaMaxBlobDistance(island3ID, 10.0);
	rmSetAreaCoherence(island3ID, 0.5);
	rmSetAreaBaseHeight(island3ID, 2.0);
	rmSetAreaSmoothDistance(island3ID, 10.0);
	rmSetAreaHeightBlend(island3ID, 2.0);
	rmSetAreaTerrainType(island3ID, "SandA");
	rmAddAreaConstraint(island3ID, shallowConstraintSW);

	// Text
	rmSetStatusText("",0.7);

	// Create northeastern peninsula
	// Part 1
	int island4ID=rmCreateArea("peninsula 4");
	rmSetAreaSize(island4ID, rmXMetersToFraction(size1), rmZMetersToFraction(size1));
	rmSetAreaLocation(island4ID, loc4x, loc4z);
	rmAddAreaToClass(island4ID, classLand);
	rmSetAreaWarnFailure(island4ID, false);
	rmSetAreaMinBlobs(island4ID, 1);
	rmSetAreaMaxBlobs(island4ID, 5);
	rmSetAreaMinBlobDistance(island4ID, 5.0);
	rmSetAreaMaxBlobDistance(island4ID, 10.0);
	rmSetAreaCoherence(island4ID, 0.5);
	rmSetAreaBaseHeight(island4ID, 2.0);
	rmSetAreaSmoothDistance(island4ID, 10.0);
	rmSetAreaHeightBlend(island4ID, 2.0);
	rmSetAreaTerrainType(island4ID, "SandA");
	rmAddAreaConstraint(island4ID, shallowConstraintNE);

	// Part 2
	int island5ID=rmCreateArea("peninsula 5");
	rmSetAreaSize(island5ID, rmXMetersToFraction(size2), rmZMetersToFraction(size2));
	rmSetAreaLocation(island5ID, loc5x, loc5z);
	rmAddAreaToClass(island5ID, classLand);
	rmSetAreaWarnFailure(island5ID, false);
	rmSetAreaMinBlobs(island5ID, 1);
	rmSetAreaMaxBlobs(island5ID, 5);
	rmSetAreaMinBlobDistance(island5ID, 5.0);
	rmSetAreaMaxBlobDistance(island5ID, 10.0);
	rmSetAreaCoherence(island5ID, 0.5);
	rmSetAreaBaseHeight(island5ID, 2.0);
	rmSetAreaSmoothDistance(island5ID, 10.0);
	rmSetAreaHeightBlend(island5ID, 2.0);
	rmSetAreaTerrainType(island5ID, "SandA");
	rmAddAreaConstraint(island5ID, shallowConstraintNE);

	// Part 3
	int island6ID=rmCreateArea("peninsula 6");
	rmSetAreaSize(island6ID, rmXMetersToFraction(size3), rmZMetersToFraction(size3));
	rmSetAreaLocation(island6ID, loc6x, loc6z);
	rmAddAreaToClass(island6ID, classLand);
	rmSetAreaWarnFailure(island6ID, false);
	rmSetAreaMinBlobs(island6ID, 1);
	rmSetAreaMaxBlobs(island6ID, 5);
	rmSetAreaMinBlobDistance(island6ID, 5.0);
	rmSetAreaMaxBlobDistance(island6ID, 10.0);
	rmSetAreaCoherence(island6ID, 0.5);
	rmSetAreaBaseHeight(island6ID, 2.0);
	rmSetAreaSmoothDistance(island6ID, 10.0);
	rmSetAreaHeightBlend(island6ID, 2.0);
	rmSetAreaTerrainType(island6ID, "SandA");
	rmAddAreaConstraint(island6ID, shallowConstraintNE);

	rmBuildAllAreas();

	// Text
	rmSetStatusText("",0.6);

	// Oceans
	float ocean1x=0;
	float ocean1z=0;
	float ocean2x=0;
	float ocean2z=0;

	if(skew==1)
	{
		ocean1x=0.5;
		ocean1z=1.0;
		ocean2x=0.5;
		ocean2z=0.0;
	}
	else
	{
		ocean1x=1.0;
		ocean1z=0.5;
		ocean2x=0.0;
		ocean2z=0.5;
	}

	// Big ocean
	int ocean1ID=rmCreateArea("mediterranean");
	rmSetAreaSize(ocean1ID, 0.25, 0.25);
	rmSetAreaLocation(ocean1ID, ocean1x, ocean1z);
	rmAddAreaToClass(ocean1ID, classOcean);
	rmSetAreaWarnFailure(ocean1ID, false);
	rmSetAreaCoherence(ocean1ID, 1.0);
	rmSetAreaBaseHeight(ocean1ID, -3.0);
	rmSetAreaSmoothDistance(ocean1ID, 20.0);
	rmSetAreaHeightBlend(ocean1ID, 2.0);
	rmSetAreaWaterType(ocean1ID, "Egyptian Nile");
	rmAddAreaConstraint(ocean1ID, oceanAvoidsLand);
	rmAddAreaConstraint(ocean1ID, oceanConstraint);

	// Small ocean
	if(cNumberNonGaiaPlayers > 3)
	{
		int ocean2ID=rmCreateArea("nile");
		rmSetAreaSize(ocean2ID, 1.0, 1.0);
		rmSetAreaLocation(ocean2ID, ocean2x, ocean2z);
		rmAddAreaToClass(ocean2ID, classOcean);
		rmSetAreaWarnFailure(ocean2ID, false);
		rmSetAreaCoherence(ocean2ID, 1.0);
		rmSetAreaBaseHeight(ocean2ID, 2.0);
		rmSetAreaSmoothDistance(ocean2ID, 10);
		rmSetAreaHeightBlend(ocean2ID, 2.0);
		rmSetAreaWaterType(ocean2ID, "Egyptian Nile");
		rmAddAreaConstraint(ocean2ID, oceanAvoidsLand);
		rmAddAreaConstraint(ocean2ID, smallOceanConstraint);
	}

	rmBuildAllAreas();

	// Set up player areas.
	float playerFraction=rmAreaTilesToFraction(3500);
	for(i=1; <cNumberPlayers)
	{
		int id=rmCreateArea("Player"+i);
		rmSetPlayerArea(i, id);
		rmSetAreaSize(id, 0.05, 0.05);
		rmAddAreaToClass(id, classPlayer);
		rmSetAreaWarnFailure(id, false);
		rmSetAreaMinBlobs(id, 1);
		rmSetAreaMaxBlobs(id, 5);
		rmSetAreaMinBlobDistance(id, 5.0);
		rmSetAreaMaxBlobDistance(id, 10.0);
		rmSetAreaCoherence(id, 0.6);
		rmSetAreaBaseHeight(id, 1.0);
		rmSetAreaSmoothDistance(id, 10);
		rmSetAreaHeightBlend(id, 2);
		rmSetAreaLocPlayer(id, i);
		rmAddAreaConstraint(id, shortAvoidOcean);
	}

	rmBuildAllAreas();

	// Text
	rmSetStatusText("",0.5);

	// Settlement 1
	rmPlaceObjectDefPerPlayer(startingSettlementID, true);
	if(cNumberNonGaiaPlayers < 3)
		id=rmAddFairLoc("Settlement", false, false, rmRandFloat(55,65), rmRandFloat(70, 80), 30, 16);
	else
	{
		if(rmRandFloat(0,1)<0.5)
			id=rmAddFairLoc("Settlement", false, true, 60, 100, 40, 16);
		else
			id=rmAddFairLoc("Settlement", false, false, 60, 100, 40, 16);
	}
	rmAddFairLocConstraint(id, startingSettleConstraint);
	rmAddFairLocConstraint(id, settlementAvoidsOcean);

	// Settlement 2
	if(cNumberNonGaiaPlayers < 3)
	{
		if(rmRandFloat(0,1)<0.85)
			id=rmAddFairLoc("Settlement", true, true,  60, 110, 50, 80);
		else
			id=rmAddFairLoc("Settlement", false, false,  60, 130, 40, 30);
	}
	else
		id=rmAddFairLoc("Settlement", true, true,  60, 110, 60, 90);
	rmAddFairLocConstraint(id, startingSettleConstraint);
	rmAddFairLocConstraint(id, mediumCenterLineConstraint);
	rmAddFairLocConstraint(id, settlementAvoidsOcean);
	rmAddFairLocConstraint(id, settlementVsSettlement);

	if(rmPlaceFairLocs())
	{
		id=rmCreateObjectDef("far settlement");
		rmAddObjectDefItem(id, "Settlement", 1, 0.0);
		for(i=1; <cNumberPlayers)
		{
			for(j=0; <rmGetNumberFairLocs(i))
			{
				int settleArea = rmCreateArea("settlement area"+i +j, rmAreaID("Player"+i));
				rmSetAreaLocation(settleArea, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j));
				rmBuildArea(settleArea);
				rmPlaceObjectDefAtAreaLoc(id, i, settleArea);
			}
		}
	}

	// Text
	rmSetStatusText("",0.4);

	// Texture embellishment 1
	for(i=1; <cNumberPlayers*6)
	{
		int id1=rmCreateArea("beautification 1 "+i);
		rmSetAreaSize(id1, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id1, "EgyptianRoadA");
		rmAddAreaConstraint(id1, mediumAvoidOcean);
		rmAddAreaConstraint(id1, shallowConstraintSW);
		rmSetAreaMinBlobs(id1, 1);
		rmSetAreaMaxBlobs(id1, 5);
		rmSetAreaWarnFailure(id1, false);
		rmSetAreaMinBlobDistance(id1, 16.0);
		rmSetAreaMaxBlobDistance(id1, 40.0);
		rmSetAreaCoherence(id1, 0.6);
		rmBuildArea(id1);
	}

	 for(i=1; <cNumberPlayers*6)
	{
		int id2=rmCreateArea("more beautification 2 "+i);
		rmSetAreaSize(id2, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id2, "SandC");
		rmAddAreaConstraint(id2, mediumAvoidOcean);
		rmAddAreaConstraint(id2, shallowConstraintSW);
		rmSetAreaMinBlobs(id2, 1);
		rmSetAreaMaxBlobs(id2, 5);
		rmSetAreaWarnFailure(id2, false);
		rmSetAreaMinBlobDistance(id2, 16.0);
		rmSetAreaMaxBlobDistance(id2, 40.0);
		rmSetAreaCoherence(id2, 0.2);
		rmBuildArea(id2);
	}

	for(i=1; <cNumberPlayers*6)
	{
		int id3=rmCreateArea("even more beautification 3 "+i);
		rmSetAreaSize(id3, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id3, "SandD");
		rmAddAreaConstraint(id3, mediumAvoidOcean);
		rmAddAreaConstraint(id3, shallowConstraintSW);
		rmSetAreaMinBlobs(id3, 1);
		rmSetAreaMaxBlobs(id3, 5);
		rmSetAreaWarnFailure(id3, false);
		rmSetAreaMinBlobDistance(id3, 16.0);
		rmSetAreaMaxBlobDistance(id3, 40.0);
		rmSetAreaCoherence(id3, 0.2);
		rmBuildArea(id3);
	}

	// Texture embellishment 2
	for(i=1; <cNumberPlayers*6)
	{
		id1=rmCreateArea("beautification 4 "+i);
		rmSetAreaSize(id1, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id1, "EgyptianRoadA");
		rmAddAreaConstraint(id1, mediumAvoidOcean);
		rmAddAreaConstraint(id1, shallowConstraintNE);
		rmSetAreaMinBlobs(id1, 1);
		rmSetAreaMaxBlobs(id1, 5);
		rmSetAreaWarnFailure(id1, false);
		rmSetAreaMinBlobDistance(id1, 16.0);
		rmSetAreaMaxBlobDistance(id1, 40.0);
		rmSetAreaCoherence(id1, 0.6);
		rmBuildArea(id1);
	}

	 for(i=1; <cNumberPlayers*6)
	{
		id2=rmCreateArea("more beautification 5 "+i);
		rmSetAreaSize(id2, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id2, "SandC");
		rmAddAreaConstraint(id2, mediumAvoidOcean);
		rmAddAreaConstraint(id2, shallowConstraintNE);
		rmSetAreaMinBlobs(id2, 1);
		rmSetAreaMaxBlobs(id2, 5);
		rmSetAreaWarnFailure(id2, false);
		rmSetAreaMinBlobDistance(id2, 16.0);
		rmSetAreaMaxBlobDistance(id2, 40.0);
		rmSetAreaCoherence(id2, 0.2);
		rmBuildArea(id2);
	}

	for(i=1; <cNumberPlayers*6)
	{
		id3=rmCreateArea("even more beautification 6 "+i);
		rmSetAreaSize(id3, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id3, "SandD");
		rmAddAreaConstraint(id3, mediumAvoidOcean);
		rmAddAreaConstraint(id3, shallowConstraintNE);
		rmSetAreaMinBlobs(id3, 1);
		rmSetAreaMaxBlobs(id3, 5);
		rmSetAreaWarnFailure(id3, false);
		rmSetAreaMinBlobDistance(id3, 16.0);
		rmSetAreaMaxBlobDistance(id3, 40.0);
		rmSetAreaCoherence(id3, 0.2);
		rmBuildArea(id3);
	}

	// Text
	rmSetStatusText("",0.3);

	// Place close objects
	// Towers
	rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

	// Gold
	rmPlaceObjectDefPerPlayer(startingGoldID, false);

	// Food
	rmPlaceObjectDefPerPlayer(startingFoodID, false);

	// Pigs
	rmPlaceObjectDefPerPlayer(startingPigsID, true);

	// Straggler trees
	rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(4, 5));

	// Medium
	// Gold
	rmPlaceObjectDefPerPlayer(mediumGoldID, false);

	// Water hippos
	rmPlaceObjectDefPerPlayer(waterHippoID, false);

	// Medium hunt
	rmPlaceObjectDefPerPlayer(mediumHuntID, false);

	// Medium pigs
	rmPlaceObjectDefPerPlayer(mediumPigsID, false);

	// Predator
	rmPlaceObjectDefPerPlayer(predatorID, false);

	// Far
	// Gold
	rmPlaceObjectDefPerPlayer(farGoldID, false, 2);

	// Random hunt
	if(rmRandFloat(0,1)>0.2)
		rmPlaceObjectDefPerPlayer(randomHuntID, false);

	// Buffalos
	if(cNumberNonGaiaPlayers == 2)
		rmPlaceObjectDefPerPlayer(buffaloID, false, rmRandInt(1,2));

	// Far Pigs
	rmPlaceObjectDefPerPlayer(farPigsID, false, 2);

	// Forest
	int failCount=0;
	int numTries=15*cNumberNonGaiaPlayers;
	for(i=0; <numTries)
	{
		int forestID=rmCreateArea("forest"+i);
		rmSetAreaSize(forestID, rmAreaTilesToFraction(50), rmAreaTilesToFraction(80));
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaForestType(forestID, "palm forest");
		rmAddAreaConstraint(forestID, forestSettleConstraint);
		rmAddAreaConstraint(forestID, avoidSettlement);
		rmAddAreaConstraint(forestID, shortAvoidAll);
		rmAddAreaConstraint(forestID, shortAvoidGold);
		rmAddAreaConstraint(forestID, forestConstraint);
		rmAddAreaConstraint(forestID, mediumAvoidOcean);
		rmAddAreaToClass(forestID, classForest);
		rmSetAreaCoherence(forestID, 0.6);
		rmSetAreaBaseHeight(forestID, 1.0);
		rmSetAreaSmoothDistance(forestID, 2);
		rmSetAreaHeightBlend(forestID, 2);
		rmSetAreaMinBlobs(forestID, 3);
		rmSetAreaMaxBlobs(forestID, 7);
		rmSetAreaMinBlobDistance(forestID, 16.0);
		rmSetAreaMaxBlobDistance(forestID, 40.0);
		rmSetAreaCoherence(forestID, 0.0);

		if(rmBuildArea(forestID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3)
				break;
		}
		else
			failCount=0;
	}

	// Text
	rmSetStatusText("",0.2);

	// Relics
	rmPlaceObjectDefPerPlayer(relicID, false);

	// Random trees
	rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 20*cNumberNonGaiaPlayers);

	// Hawks
	rmPlaceObjectDefPerPlayer(farhawkID, false, 2);

	// NW fish
	int northernFishID=rmCreateObjectDef("northern fish");
	rmAddObjectDefItem(northernFishID, "fish - salmon", 3, 8.0);
	rmSetObjectDefMinDistance(northernFishID, 0.0);
	rmSetObjectDefMaxDistance(northernFishID, 90.0);
	rmAddObjectDefConstraint(northernFishID, longFishVsFishID);
	rmAddObjectDefConstraint(northernFishID, fishNearShore);
	rmAddObjectDefConstraint(northernFishID, fishLand);
	rmAddObjectDefConstraint(northernFishID, oceanConstraint);
	rmAddObjectDefConstraint(northernFishID, tinyCenterLineConstraint);
	rmAddObjectDefConstraint(northernFishID, longEdgeConstraint);
	rmPlaceObjectDefPerPlayer(northernFishID, false, 4);

	// NW extra fish
		int extraFishID=rmCreateObjectDef("extra fish");
		rmAddObjectDefItem(extraFishID, "fish - salmon", 3, 8.0);
		rmSetObjectDefMinDistance(extraFishID, 0.0);
		rmSetObjectDefMaxDistance(extraFishID, rmXFractionToMeters(1.0));
		rmAddObjectDefConstraint(extraFishID, extraFishVsFishID);
		rmAddObjectDefConstraint(extraFishID, fishLand);
		rmAddObjectDefConstraint(extraFishID, oceanConstraint);
		rmAddObjectDefConstraint(extraFishID, edgeConstraint);
		if(cNumberNonGaiaPlayers > 5)
			rmPlaceObjectDefAtLoc(extraFishID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);
		else if(cNumberNonGaiaPlayers > 3)
			rmPlaceObjectDefAtLoc(extraFishID, 0, 0.5, 0.5, 2*cNumberNonGaiaPlayers);
		else
			rmPlaceObjectDefAtLoc(extraFishID, 0, 0.5, 0.5, cNumberNonGaiaPlayers);


	// SE fish
	if(cNumberNonGaiaPlayers > 3)
	{
		int southernFishID=rmCreateObjectDef("southern fish");
		rmAddObjectDefItem(southernFishID, "fish - salmon", 3, 8.0);
		rmSetObjectDefMinDistance(southernFishID, 0.0);
		rmSetObjectDefMaxDistance(southernFishID, 125.0);
		rmAddObjectDefConstraint(southernFishID, shortFishVsFishID);
		rmAddObjectDefConstraint(southernFishID, fishLand);
		rmAddObjectDefConstraint(southernFishID, smallOceanConstraint);
		rmPlaceObjectDefPerPlayer(southernFishID, false, cNumberNonGaiaPlayers-2);
	}

	// Text
	rmSetStatusText("",0.1);

	// Embellishment
	int rockID=rmCreateObjectDef("rock small");
	rmAddObjectDefItem(rockID, "rock sandstone small", 3, 2.0);
	rmAddObjectDefItem(rockID, "rock limestone sprite", 4, 3.0);
	rmSetObjectDefMinDistance(rockID, 0.0);
	rmSetObjectDefMaxDistance(rockID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(rockID, shortAvoidAll);
	rmAddObjectDefConstraint(rockID, shortAvoidOcean);
	rmPlaceObjectDefAtLoc(rockID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);

	int rockID2=rmCreateObjectDef("rock sprite");
	rmAddObjectDefItem(rockID2, "rock sandstone sprite", 1, 0.0);
	rmSetObjectDefMinDistance(rockID2, 0.0);
	rmSetObjectDefMaxDistance(rockID2, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(rockID2, shortAvoidAll);
	rmAddObjectDefConstraint(rockID2, shortAvoidOcean);
	rmPlaceObjectDefAtLoc(rockID2, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);

	int papyrusID=rmCreateObjectDef("lone papyrus");
	rmAddObjectDefItem(papyrusID, "papyrus", 3, 2.0);
	rmSetObjectDefMinDistance(papyrusID, 0.0);
	rmSetObjectDefMaxDistance(papyrusID, 600);
	rmAddObjectDefConstraint(papyrusID, shortAvoidAll);
	rmAddObjectDefConstraint(papyrusID, nearShore);
	rmPlaceObjectDefAtLoc(papyrusID, 0, 0.5, 0.5, 6*cNumberNonGaiaPlayers);

	int papyrus2ID=rmCreateObjectDef("grouped papyrus");
	rmAddObjectDefItem(papyrus2ID, "papyrus", 5, 7.0);
	rmSetObjectDefMinDistance(papyrus2ID, 0.0);
	rmSetObjectDefMaxDistance(papyrus2ID, 600);
	rmAddObjectDefConstraint(papyrus2ID, shortAvoidAll);
	rmAddObjectDefConstraint(papyrus2ID, rmCreateTerrainMaxDistanceConstraint("papyrus near shore", "land", true, 4.0));
	rmPlaceObjectDefAtLoc(papyrus2ID, 0, 0.5, 0.5, 2*cNumberNonGaiaPlayers);

	int lilyID=rmCreateObjectDef("pads");
	rmAddObjectDefItem(lilyID, "water lilly", 4, 2.0);
	rmSetObjectDefMinDistance(lilyID, 0.0);
	rmSetObjectDefMaxDistance(lilyID, 600);
	rmAddObjectDefConstraint(lilyID, shortAvoidAll);
	rmPlaceObjectDefAtLoc(lilyID, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);

	int lily2ID=rmCreateObjectDef("more pads");
	rmAddObjectDefItem(lily2ID, "water lilly", 1, 0.0);
	rmSetObjectDefMinDistance(lily2ID, 0.0);
	rmSetObjectDefMaxDistance(lily2ID, 600);
	rmAddObjectDefConstraint(lily2ID, shortAvoidAll);
	rmPlaceObjectDefAtLoc(lily2ID, 0, 0.5, 0.5, 8*cNumberNonGaiaPlayers);

	int bushID=rmCreateObjectDef("big bush patch");
	rmAddObjectDefItem(bushID, "bush", 4, 3.0);
	rmSetObjectDefMinDistance(bushID, 0.0);
	rmSetObjectDefMaxDistance(bushID, 600);
	rmAddObjectDefConstraint(bushID, shortAvoidAll);
	rmPlaceObjectDefAtLoc(bushID, 0, 0.5, 0.5, 10*cNumberNonGaiaPlayers);

	int bush2ID=rmCreateObjectDef("small bush patch");
	rmAddObjectDefItem(bush2ID, "bush", 3, 2.0);
	rmAddObjectDefItem(bush2ID, "rock sandstone sprite", 1, 2.0);
	rmSetObjectDefMinDistance(bush2ID, 0.0);
	rmSetObjectDefMaxDistance(bush2ID, 600);
	rmAddObjectDefConstraint(bush2ID, shortAvoidAll);
	rmPlaceObjectDefAtLoc(bush2ID, 0, 0.5, 0.5, 8*cNumberNonGaiaPlayers);

	int riverBushID=rmCreateObjectDef("bushs by river");
	rmAddObjectDefItem(riverBushID, "bush", 3, 3.0);
	rmAddObjectDefItem(riverBushID, "grass", 7, 8.0);
	rmSetObjectDefMinDistance(riverBushID, 0.0);
	rmSetObjectDefMaxDistance(riverBushID, 600);
	rmAddObjectDefConstraint(riverBushID, shortAvoidAll);
	rmAddObjectDefConstraint(riverBushID, nearShore);
	rmPlaceObjectDefAtLoc(riverBushID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);

	// Text
	rmSetStatusText("",0.01);

	// Triggers
	// Disable bolt for a minute
	int zeusPlayer=0;
	for(i=1; <cNumberPlayers)
	{
		if(rmGetPlayerCiv(i) == cCivZeus)
		{
			zeusPlayer=1;
			rmCreateTrigger("disable bolt"+i);
			rmSetTriggerActive(true);
			rmSetTriggerPriority(5);
			rmAddTriggerCondition("Timer");
			rmSetTriggerConditionParamInt("Param1", 1);
			rmAddTriggerEffect("Player Kill All God Powers");
			rmSetTriggerEffectParamInt("Player", i);

			rmCreateTrigger("grant bolt"+i);
			rmSetTriggerActive(true);
			rmSetTriggerPriority(5);
			rmAddTriggerCondition("Timer");
			rmSetTriggerConditionParamInt("Param1", 60);
			rmAddTriggerEffect("Grant God Power");
			rmSetTriggerEffectParamInt("PlayerID", i);
			rmSetTriggerEffectParam("PowerName", "bolt");
			rmSetTriggerEffectParamInt("Count", 1);
		}
	}

	// Notify players 1
	if(cNumberTeams > 2)
	{
		rmCreateTrigger("team notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
	else if(cNumberNonGaiaPlayers > 6)
	{
		rmCreateTrigger("player notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}

	// Notify players 2
	if(zeusPlayer==1)
	{
		rmCreateTrigger("bolt notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: Zeus' Bolt power has been disabled for the first minute.", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
}