// Painted Desert 1.0
// Script: RR
// Concept: Bubble/RR

// Main entry point for random map script
void main(void)
{
	// Text
	rmSetStatusText("",1.0);

	// Set size
	int playerTiles=9600;
	int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
	int skew=rmRandInt(1,2);
	if(cNumberNonGaiaPlayers > 4)
	{
		if(skew==1)
		{
			rmSetMapSize(size, size*1.1);
			rmEchoInfo("Map size="+size+"m x "+size*1.1+"m");
		}
		else
		{
			rmSetMapSize(size*1.1, size);
			rmEchoInfo("Map size="+size*1.1+"m x "+size+"m");
		}
	}
	else
	{
		rmSetMapSize(size, size);
		rmEchoInfo("Map size="+size+"m x "+size+"m");
	}

	// Init map
	rmSetSeaLevel(-1.0);
	rmTerrainInitialize("SandC", 0.0);
	rmSetLightingSet("Alfheim");

	// Define clssses
	int classCliff=rmDefineClass("cliff");
	int classPlayer=rmDefineClass("player");
	int classForest=rmDefineClass("forest");
	rmDefineClass("starting settlement");
	rmDefineClass("center line");

	// Constraints
	// General
	int divisionConstraint=0;
	if(skew==1)
		divisionConstraint=rmCreateBoxConstraint("divide map", 0.48, 0.0, 0.52, 1.0, 0.01);
	else
		divisionConstraint=rmCreateBoxConstraint("divide map", 0.0, 0.48, 1.0, 0.52, 0.01);
	int edgeConstraint=rmCreateBoxConstraint("edge constraint", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8));
	int shortAvoidAll=rmCreateTypeDistanceConstraint("short avoid all", "all", 3.0);
	int shortCenterLineConstraint=rmCreateClassDistanceConstraint("stay a bit away from center line", rmClassID("center line"), 12.5);
	int mediumCenterLineConstraint=rmCreateClassDistanceConstraint("stay away from center line", rmClassID("center line"), 20.0);
	int farCenterLineConstraint=rmCreateClassDistanceConstraint("stay far away from center line", rmClassID("center line"), 80.0);
	int avoidPlayer=rmCreateClassDistanceConstraint("avoid player areas", classPlayer, 10.0);

	// Terrain
	int waterNWConstraint=0;
	int waterSEConstraint=0;
	if(skew==1)
	{
		waterNWConstraint=rmCreateBoxConstraint("water NW limit", 0.0, 0.9, 1.0, 1.0, 0.01);
		waterSEConstraint=rmCreateBoxConstraint("water SE limit", 0.0, 0.0, 1.0, 0.1, 0.01);
	}
	else
	{
		waterNWConstraint=rmCreateBoxConstraint("water NW limit", 0.9, 0.0, 1.0, 1.0, 0.01);
		waterSEConstraint=rmCreateBoxConstraint("water SE limit", 0.0, 0.0, 0.1, 1.0, 0.01);
	}
	int farCliffConstraint=rmCreateClassDistanceConstraint("long avoid cliff", rmClassID("cliff"), 30.0);
	int mediumCliffConstraint=rmCreateClassDistanceConstraint("medium avoid cliff", rmClassID("cliff"), 10.0);
	int shortCliffConstraint=rmCreateClassDistanceConstraint("short avoid cliff", rmClassID("cliff"), 5.0);
	int cliffPlayerConstraint=rmCreateClassDistanceConstraint("cliffs stay away from players", classPlayer, 5.0);
	int cliffEdgeConstraint=rmCreateBoxConstraint("cliffs avoid edge of the map", rmXTilesToFraction(32), rmZTilesToFraction(32), 1.0-rmXTilesToFraction(32), 1.0-rmZTilesToFraction(32));
	int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "land", false, 4.0);

	// Settlements
	int startingSettleConstraint=rmCreateClassDistanceConstraint("avoid starting settlement", rmClassID("starting settlement"), 30.0);
	int shortStartingSettleConstraint=rmCreateClassDistanceConstraint("short avoid starting settlement", rmClassID("starting settlement"), 20.0);
	int settlementVsSettlement=rmCreateTypeDistanceConstraint("TCs avoid TCs", "AbstractSettlement", 50.0);
	int avoidSettlement=rmCreateTypeDistanceConstraint("avoid TCs", "AbstractSettlement", 15.0);

	// Gold
	int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 8.0);
	int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
	int farAvoidGold=rmCreateTypeDistanceConstraint("far avoid gold", "gold", 40.0);

	// Food
	int avoidHunt=rmCreateTypeDistanceConstraint("avoid huntable", "huntable", 40.0);
	int shortAvoidFood=rmCreateTypeDistanceConstraint("short avoid huntable", "food", 8.0);
	int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 40.0);
	int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 40.0);

	// Wood
	int shortForestConstraint=rmCreateClassDistanceConstraint("short avoid forest", rmClassID("forest"), 5.0);
	int mediumForestConstraint=rmCreateClassDistanceConstraint("medium avoid forest", rmClassID("forest"), 12.5);
	int farForestConstraint=rmCreateClassDistanceConstraint("far avoid forest", rmClassID("forest"), 20.0);
	int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);

	// Buildings
	int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 15.0);
	int avoidTower=rmCreateTypeDistanceConstraint("avoid tower", "tower", 25.0);
	int shortAvoidTower=rmCreateTypeDistanceConstraint("short avoid tower", "tower", 10.0);

	// Text
	rmSetStatusText("",0.9);

	// Define objects
	// Starting settlement
	int startingSettlementID=rmCreateObjectDef("Starting settlement");
	rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
	rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
	rmSetObjectDefMinDistance(startingSettlementID, 0.0);
	rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

	// Towers
	int startingTowerID=rmCreateObjectDef("starting tower");
	rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
	rmSetObjectDefMinDistance(startingTowerID, 22.0);
	rmSetObjectDefMaxDistance(startingTowerID, 24.0);
	rmAddObjectDefConstraint(startingTowerID, avoidTower);

	// Gold
	int startingGoldID=rmCreateObjectDef("starting gold");
	rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
	rmSetObjectDefMinDistance(startingGoldID, 20.0);
	rmSetObjectDefMaxDistance(startingGoldID, 25.0);
	rmAddObjectDefConstraint(startingGoldID, shortAvoidAll);
	rmAddObjectDefConstraint(startingGoldID, shortAvoidTower);

	// Hunt
	int startingRhinoID=rmCreateObjectDef("starting rhinos");
	rmAddObjectDefItem(startingRhinoID, "Rhinocerous", 2, 3.0);
	rmSetObjectDefMinDistance(startingRhinoID, 27.5);
	rmSetObjectDefMaxDistance(startingRhinoID, 27.5);
	rmAddObjectDefConstraint(startingRhinoID, shortAvoidAll);

	// Berries
	int startingFoodID=rmCreateObjectDef("close food");
	rmAddObjectDefItem(startingFoodID, "berry bush", rmRandInt(5,7), 4.0);
	rmSetObjectDefMinDistance(startingFoodID, 20.0);
	rmSetObjectDefMaxDistance(startingFoodID, 25.0);
	rmAddObjectDefConstraint(startingFoodID, shortAvoidAll);
	rmAddObjectDefConstraint(startingFoodID, shortAvoidGold);

	// Pigs
	int startingPigsID=rmCreateObjectDef("close pigs");
	rmAddObjectDefItem(startingPigsID, "pig", 2, 2.0);
	rmSetObjectDefMinDistance(startingPigsID, 25.0);
	rmSetObjectDefMaxDistance(startingPigsID, 30.0);
	rmAddObjectDefConstraint(startingPigsID, shortAvoidAll);

	// Straggler trees
	int stragglerTreeID=rmCreateObjectDef("straggler tree");
	rmAddObjectDefItem(stragglerTreeID, "palm", 1, 0.0);
	rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
	rmSetObjectDefMaxDistance(stragglerTreeID, 20.0);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidGold);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidAll);

	// Medium objects
	// Medium gold
	int mediumGoldID=rmCreateObjectDef("medium gold");
	rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(mediumGoldID, 50.0);
	rmSetObjectDefMaxDistance(mediumGoldID, 65.0);
	rmAddObjectDefConstraint(mediumGoldID, edgeConstraint);
	rmAddObjectDefConstraint(mediumGoldID, mediumForestConstraint);
	rmAddObjectDefConstraint(mediumGoldID, farCenterLineConstraint);
	rmAddObjectDefConstraint(mediumGoldID, avoidSettlement);
	rmAddObjectDefConstraint(mediumGoldID, mediumCliffConstraint);
	rmAddObjectDefConstraint(mediumGoldID, avoidGold);
	rmAddObjectDefConstraint(mediumGoldID, startingSettleConstraint);

	// Gazelles
	int gazelleID=rmCreateObjectDef("gazelles");
	rmAddObjectDefItem(gazelleID, "gazelle", rmRandInt(6,7), 3.0);
	rmSetObjectDefMinDistance(gazelleID, 70.0);
	rmSetObjectDefMaxDistance(gazelleID, 80.0);
	rmAddObjectDefConstraint(gazelleID, shortAvoidAll);
	rmAddObjectDefConstraint(gazelleID, edgeConstraint);
	rmAddObjectDefConstraint(gazelleID, farCenterLineConstraint);
	rmAddObjectDefConstraint(gazelleID, avoidHunt);
	rmAddObjectDefConstraint(gazelleID, shortCliffConstraint);
	rmAddObjectDefConstraint(gazelleID, startingSettleConstraint);

	// Giraffes
	int giraffeID=rmCreateObjectDef("giraffes");
	rmAddObjectDefItem(giraffeID, "giraffe", rmRandInt(3,5), 3.0);
	rmSetObjectDefMinDistance(giraffeID, 60.0);
	rmSetObjectDefMaxDistance(giraffeID, 70.0);
	rmAddObjectDefConstraint(giraffeID, shortAvoidAll);
	rmAddObjectDefConstraint(giraffeID, edgeConstraint);
	if(cNumberNonGaiaPlayers == 2)
		rmAddObjectDefConstraint(giraffeID, shortCenterLineConstraint);
	else
		rmAddObjectDefConstraint(giraffeID, farCenterLineConstraint);
	rmAddObjectDefConstraint(giraffeID, avoidHunt);
	rmAddObjectDefConstraint(giraffeID, mediumCliffConstraint);

	// Rhinos
	int rhinoID=rmCreateObjectDef("rhinos");
	rmAddObjectDefItem(rhinoID, "rhinocerous", rmRandInt(2,3), 3.0);
	rmSetObjectDefMinDistance(rhinoID, 60.0);
	rmSetObjectDefMaxDistance(rhinoID, 70.0);
	rmAddObjectDefConstraint(rhinoID, shortAvoidAll);
	rmAddObjectDefConstraint(rhinoID, cliffEdgeConstraint);
	rmAddObjectDefConstraint(rhinoID, avoidHunt);
	rmAddObjectDefConstraint(rhinoID, mediumCliffConstraint);
	rmAddObjectDefConstraint(rhinoID, startingSettleConstraint);

	// Medium pigs
	int mediumPigsID=rmCreateObjectDef("medium pigs");
	rmAddObjectDefItem(mediumPigsID, "pig", 2, 2.0);
	rmSetObjectDefMinDistance(mediumPigsID, 55.0);
	rmSetObjectDefMaxDistance(mediumPigsID, 65.0);
	rmAddObjectDefConstraint(mediumPigsID, avoidHerdable);
	rmAddObjectDefConstraint(mediumPigsID, shortAvoidAll);
	rmAddObjectDefConstraint(mediumPigsID, edgeConstraint);
	rmAddObjectDefConstraint(mediumPigsID, shortCliffConstraint);
	rmAddObjectDefConstraint(mediumPigsID, mediumCenterLineConstraint);

	// Predators
	int predatorID=rmCreateObjectDef("predator");
	rmAddObjectDefItem(predatorID, "lion", 2, 4.0);
	rmSetObjectDefMinDistance(predatorID, 65.0);
	rmSetObjectDefMaxDistance(predatorID, 80.0);
	rmAddObjectDefConstraint(predatorID, shortAvoidAll);
	rmAddObjectDefConstraint(predatorID, edgeConstraint);
	rmAddObjectDefConstraint(predatorID, avoidPredator);
	rmAddObjectDefConstraint(predatorID, shortCliffConstraint);
	rmAddObjectDefConstraint(predatorID, avoidPlayer);

	// Far objects
	// Far gold
	int farGoldID=rmCreateObjectDef("far gold");
	rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
	if(cNumberNonGaiaPlayers < 5)
	{
		rmSetObjectDefMinDistance(farGoldID, 60.0);
		rmSetObjectDefMaxDistance(farGoldID, 100.0);
	}
	else
	{
		rmSetObjectDefMinDistance(farGoldID, 80.0);
		rmSetObjectDefMaxDistance(farGoldID, 120.0);
	}
	rmAddObjectDefConstraint(farGoldID, edgeConstraint);
	rmAddObjectDefConstraint(farGoldID, farAvoidGold);
	rmAddObjectDefConstraint(farGoldID, avoidSettlement);
	rmAddObjectDefConstraint(farGoldID, mediumForestConstraint);
	rmAddObjectDefConstraint(farGoldID, mediumCliffConstraint);
	rmAddObjectDefConstraint(farGoldID, startingSettleConstraint);
	if(cNumberNonGaiaPlayers == 2)
		rmAddObjectDefConstraint(farGoldID, mediumCenterLineConstraint);

	// Far pigs
	int farPigsID=rmCreateObjectDef("far pigs");
	rmAddObjectDefItem(farPigsID, "pig", 2, 2.0);
	rmSetObjectDefMinDistance(farPigsID, 80.0);
	rmSetObjectDefMaxDistance(farPigsID, 110.0);
	rmAddObjectDefConstraint(farPigsID, avoidHerdable);
	rmAddObjectDefConstraint(farPigsID, shortAvoidAll);
	rmAddObjectDefConstraint(farPigsID, edgeConstraint);
	rmAddObjectDefConstraint(farPigsID, shortCliffConstraint);
	rmAddObjectDefConstraint(farPigsID, shortForestConstraint);

	// Random Trees
	int randomTreeID=rmCreateObjectDef("random tree");
	rmAddObjectDefItem(randomTreeID, "palm", 1, 0.0);
	rmSetObjectDefMinDistance(randomTreeID, 0.0);
	rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
	rmAddObjectDefConstraint(randomTreeID, shortAvoidAll);
	rmAddObjectDefConstraint(randomTreeID, shortCliffConstraint);
	rmAddObjectDefConstraint(randomTreeID, shortStartingSettleConstraint);

	// Relics
	int relicID=rmCreateObjectDef("relics");
	rmAddObjectDefItem(relicID, "relic", 1, 0.0);
	rmSetObjectDefMinDistance(relicID, 70.0);
	rmSetObjectDefMaxDistance(relicID, rmXFractionToMeters(0.3));
	rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 50.0));
	rmAddObjectDefConstraint(relicID, shortAvoidAll);
	rmAddObjectDefConstraint(relicID, shortCliffConstraint);
	rmAddObjectDefConstraint(relicID, edgeConstraint);
	rmAddObjectDefConstraint(relicID, startingSettleConstraint);

	// Birds
	int farhawkID=rmCreateObjectDef("far hawks");
	rmAddObjectDefItem(farhawkID, "vulture", 1, 0.0);
	rmSetObjectDefMinDistance(farhawkID, 0.0);
	rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

	// Text
	rmSetStatusText("",0.8);

	// Player placement
	if(cNumberTeams < 3)
	{
		if(cNumberNonGaiaPlayers == 2)
		{
			if(skew==1)
				rmPlacePlayersLine(0.2, 0.5, 0.8, 0.5);
			else
				rmPlacePlayersLine(0.5, 0.2, 0.5, 0.8);
		}
		else if(cNumberNonGaiaPlayers < 5)
		{
			if(skew==1)
			{
				if(rmRandFloat(0,1)<0.5)
				{
					rmSetPlacementTeam(0);
					rmPlacePlayersLine(0.225, 0.325, 0.225, 0.675, 0.01);
					rmSetPlacementTeam(1);
					rmPlacePlayersLine(0.775, 0.325, 0.775, 0.675, 0.01);
				}
				else
				{
					rmSetPlacementTeam(0);
					rmPlacePlayersLine(0.775, 0.325, 0.775, 0.675, 0.01);
					rmSetPlacementTeam(1);
					rmPlacePlayersLine(0.225, 0.325, 0.225, 0.675, 0.01);
				}
			}
			else
			{
				if(rmRandFloat(0,1)<0.5)
				{
					rmSetPlacementTeam(0);
					rmPlacePlayersLine(0.325, 0.225, 0.675, 0.225, 0.01);
					rmSetPlacementTeam(1);
					rmPlacePlayersLine(0.325, 0.775, 0.675, 0.775, 0.01);
				}
				else
				{
					rmSetPlacementTeam(0);
					rmPlacePlayersLine(0.325, 0.775, 0.675, 0.775, 0.01);
					rmSetPlacementTeam(1);
					rmPlacePlayersLine(0.325, 0.225, 0.675, 0.225, 0.01);
				}
			}
		}
		else
		{
			if(skew==1)
			{
				if(rmRandFloat(0,1)<0.5)
				{
					rmSetPlacementTeam(0);
					rmSetPlacementSection(0.63, 0.87);
					rmPlacePlayersCircular(0.31, 0.31, rmDegreesToRadians(2.0));
					rmSetPlacementTeam(1);
					rmSetPlacementSection(0.13, 0.37);
					rmPlacePlayersCircular(0.31, 0.31, rmDegreesToRadians(2.0));
				}
				else
				{
					rmSetPlacementTeam(0);
					rmSetPlacementSection(0.13, 0.37);
					rmPlacePlayersCircular(0.31, 0.31, rmDegreesToRadians(2.0));
					rmSetPlacementTeam(1);
					rmSetPlacementSection(0.63, 0.87);
					rmPlacePlayersCircular(0.31, 0.31, rmDegreesToRadians(2.0));
				}
			}
			else
			{
				if(rmRandFloat(0,1)<0.5)
				{
					rmSetPlacementTeam(0);
					rmSetPlacementSection(0.38, 0.62);
					rmPlacePlayersCircular(0.31, 0.31, rmDegreesToRadians(2.0));
					rmSetPlacementTeam(1);
					rmSetPlacementSection(0.88, 0.12);
					rmPlacePlayersCircular(0.31, 0.31, rmDegreesToRadians(2.0));
				}
				else
				{
					rmSetPlacementTeam(0);
					rmSetPlacementSection(0.88, 0.12);
					rmPlacePlayersCircular(0.31, 0.31, rmDegreesToRadians(2.0));
					rmSetPlacementTeam(1);
					rmSetPlacementSection(0.38, 0.62);
					rmPlacePlayersCircular(0.31, 0.31, rmDegreesToRadians(2.0));
				}
			}
		}
	}
	else
		rmPlacePlayersSquare(0.3, 0.3, rmDegreesToRadians(5.0));
		// Impossible to get a fair placement with 3+ teams

	// Divide map
	int divisionID=rmCreateArea("division");
	rmSetAreaSize(divisionID, 1, 1);
	rmSetAreaLocation(divisionID, 0.5, 0.5);
	rmAddAreaToClass(divisionID, rmClassID("center line"));
	rmSetAreaWarnFailure(divisionID, false);
	rmAddAreaConstraint(divisionID, divisionConstraint);
	rmBuildArea(divisionID);

	// Set up player areas
	float playerFraction=rmAreaTilesToFraction(1600);
	for(h=1; <cNumberPlayers)
	{
		int id=rmCreateArea("Player"+h);
		rmSetPlayerArea(h, id);
		rmSetAreaSize(id, 0.9*playerFraction, 1.1*playerFraction);
		rmAddAreaToClass(id, classPlayer);
		rmSetAreaWarnFailure(id, false);
		rmSetAreaMinBlobs(id, 1);
		rmSetAreaMaxBlobs(id, 1);
		rmSetAreaCoherence(id, 1.0);
		rmSetAreaLocPlayer(id, h);
	}

	rmBuildAllAreas();

	// Text
	rmSetStatusText("",0.7);

	// Water
	// NW
	int waterNW=rmCreateArea("water NW");
	rmSetAreaSize(waterNW, 1.0, 1.0);
	if(skew==1)
		rmSetAreaLocation(waterNW, 0.5, 1);
	else
		rmSetAreaLocation(waterNW, 1, 0.5);
	rmSetAreaWarnFailure(waterNW, false);
	rmSetAreaWaterType(waterNW, "Egyptian Nile");
	rmAddAreaConstraint(waterNW, waterNWConstraint);
	rmSetAreaBaseHeight(waterNW, -1.0);
	rmSetAreaSmoothDistance(waterNW, 10.0);
	rmSetAreaHeightBlend(waterNW, 2.0);
	rmBuildArea(waterNW);

	// SE
	int waterSW=rmCreateArea("water SW");
	rmSetAreaSize(waterSW, 1.0, 1.0);
	if(skew==1)
		rmSetAreaLocation(waterSW, 0.5, 0.0);
	else
		rmSetAreaLocation(waterSW, 0.0, 0.5);
	rmSetAreaWarnFailure(waterSW, false);
	rmSetAreaWaterType(waterSW, "Egyptian Nile");
	rmAddAreaConstraint(waterSW, waterSEConstraint);
	rmSetAreaBaseHeight(waterSW, -1.0);
	rmSetAreaSmoothDistance(waterSW, 10.0);
	rmSetAreaHeightBlend(waterSW, 2.0);
	rmBuildArea(waterSW);

	// Side forests
	int forestID=0;
	int numTries=11;
	int failCount=0;

	// NW
	for(l=1; <numTries)
	{
		forestID=rmCreateArea("forest NW"+l);
		if(cNumberNonGaiaPlayers == 2)
			rmSetAreaSize(forestID, 0.0075, 0.0075);
		else if(cNumberNonGaiaPlayers < 5)
			rmSetAreaSize(forestID, 0.0072, 0.0072);
		else
			rmSetAreaSize(forestID, 0.006, 0.006);
		if(skew==1)
			rmSetAreaLocation(forestID, 0.1*l-0.05, 0.9);
		else
			rmSetAreaLocation(forestID, 0.9, 0.1*l-0.05);
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaForestType(forestID, "palm forest");
		rmSetAreaBaseHeight(forestID, 0.0);
		rmAddAreaToClass(forestID, classForest);
		rmSetAreaSmoothDistance(forestID, 10.0);
		rmSetAreaHeightBlend(forestID, 2.0);
		rmBuildArea(forestID);
	}

	// SE
	for(m=1; <numTries)
	{
		forestID=rmCreateArea("forest SE"+m);
		if(cNumberNonGaiaPlayers == 2)
			rmSetAreaSize(forestID, 0.0075, 0.0075);
		else if(cNumberNonGaiaPlayers < 5)
			rmSetAreaSize(forestID, 0.0072, 0.0072);
		else
			rmSetAreaSize(forestID, 0.006, 0.006);
		if(skew==1)
			rmSetAreaLocation(forestID, 0.1*m-0.05, 0.1);
		else
			rmSetAreaLocation(forestID, 0.1, 0.1*m-0.05);
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaForestType(forestID, "palm forest");
		rmSetAreaBaseHeight(forestID, 0.0);
		rmAddAreaToClass(forestID, classForest);
		rmSetAreaSmoothDistance(forestID, 10.0);
		rmSetAreaHeightBlend(forestID, 2.0);
		rmBuildArea(forestID);
	}

	// Text
	rmSetStatusText("",0.6);

	// Settlement 1
	rmPlaceObjectDefPerPlayer(startingSettlementID, true);
	if(cNumberNonGaiaPlayers < 3)
	{
		id=rmAddFairLoc("Settlement", false, true, 60, 80, 24, 16);
		rmAddFairLocConstraint(id, shortForestConstraint);
	}
	else if(cNumberNonGaiaPlayers < 5)
	{
		if(rmRandFloat(0,1)<0.5)
			id=rmAddFairLoc("Settlement", false, true, 40, 75, 24, 16);
		else
			id=rmAddFairLoc("Settlement", false, false, 60, 80, 36, 16);
		rmAddFairLocConstraint(id, mediumForestConstraint);
	}
	else
	{
		id=rmAddFairLoc("Settlement", false, false, 55, 75, 36, 20);
		rmAddFairLocConstraint(id, mediumForestConstraint);
	}
	rmAddFairLocConstraint(id, startingSettleConstraint);

	if(rmPlaceFairLocs())
	{
		id=rmCreateObjectDef("close settlement");
		rmAddObjectDefItem(id, "Settlement", 1, 0.0);
		for(i=1; <cNumberPlayers)
		{
			for(j=0; <rmGetNumberFairLocs(i))
			{
				int settleArea = rmCreateArea("close settlement area"+i +j, rmAreaID("Player"+i));
				rmSetAreaLocation(settleArea, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j));
				rmBuildArea(settleArea);
				rmPlaceObjectDefAtAreaLoc(id, i, settleArea);
			}
		}
	}

	rmResetFairLocs();

	// Settlement 2
	if(cNumberNonGaiaPlayers < 3)
	{
		if(rmRandFloat(0,1)<0.85)
			id=rmAddFairLoc("Settlement", true, false,  70, 85, 30, 24);
		else
			id=rmAddFairLoc("Settlement", false, false, 60, 80, 10, 24);
		rmAddFairLocConstraint(id, mediumCenterLineConstraint);
	}
	else if(cNumberNonGaiaPlayers < 5)
	{
		if(rmRandFloat(0,1)<0.5)
			id=rmAddFairLoc("Settlement", true, false,  70, 100, 70, 40);
		else
			id=rmAddFairLoc("Settlement", true, true,  70, 100, 70, 100);
	}
	else
	{
		if(rmRandFloat(0,1)<0.5)
			id=rmAddFairLoc("Settlement", true, false,  70, 90, 60, 40);
		else
			id=rmAddFairLoc("Settlement", true, true,  70, 100, 90, 60);
	}
	rmAddFairLocConstraint(id, startingSettleConstraint);
	rmAddFairLocConstraint(id, mediumForestConstraint);
	rmAddFairLocConstraint(id, settlementVsSettlement);

	if(rmPlaceFairLocs())
	{
		id=rmCreateObjectDef("far settlement");
		rmAddObjectDefItem(id, "Settlement", 1, 0.0);
		for(i=1; <cNumberPlayers)
		{
			for(j=0; <rmGetNumberFairLocs(i))
			{
				int farSettleArea = rmCreateArea("far settlement area"+i +j, rmAreaID("Player"+i));
				rmSetAreaLocation(farSettleArea, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j));
				rmBuildArea(farSettleArea);
				rmPlaceObjectDefAtAreaLoc(id, i, farSettleArea);
			}
		}
	}

	// Text
	rmSetStatusText("",0.5);

	// Cliffs
	if(cNumberNonGaiaPlayers < 3)
		numTries=3*cNumberNonGaiaPlayers+1;
	else
		numTries=2*cNumberNonGaiaPlayers+1;
	failCount=0;
	for(k=0; <numTries)
	{
		int cliffID=rmCreateArea("cliff"+k);
		rmSetAreaWarnFailure(cliffID, false);
		rmSetAreaSize(cliffID, rmAreaTilesToFraction(120), rmAreaTilesToFraction(150));
		rmSetAreaCliffType(cliffID, "Egyptian");
		rmAddAreaToClass(cliffID, classCliff);
		rmSetAreaMinBlobs(cliffID, 2);
		rmSetAreaMaxBlobs(cliffID, 2);
		rmSetAreaCliffEdge(cliffID, 1.0, 1.0, 0.0, 1.0, 0);
		rmSetAreaCliffPainting(cliffID, true, true, true, 1.5, true);
		rmSetAreaCliffHeight(cliffID, 7, 1.0, 1.0);
		rmSetAreaMinBlobDistance(cliffID, 16.0);
		rmSetAreaMaxBlobDistance(cliffID, 20.0);
		rmSetAreaCoherence(cliffID, 0.25);
		rmSetAreaSmoothDistance(cliffID, 8);
		rmSetAreaHeightBlend(cliffID, 2);
		rmAddAreaConstraint(cliffID, avoidBuildings);
		rmAddAreaConstraint(cliffID, cliffPlayerConstraint);
		rmAddAreaConstraint(cliffID, cliffEdgeConstraint);
		rmAddAreaConstraint(cliffID, farCliffConstraint);
		rmAddAreaConstraint(cliffID, farForestConstraint);
		rmAddAreaConstraint(cliffID, shortCenterLineConstraint);

		if(rmBuildArea(cliffID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3)
				break;
		}
		else
			failCount=0;
	}

	// Text
	rmSetStatusText("",0.4);

	// Texture embellishment
	for(z=1; <cNumberPlayers*15)
	{
		int id1=rmCreateArea("beautification"+z);
		rmSetAreaSize(id1, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id1, "SandB");
		rmAddAreaConstraint(id1, shortForestConstraint);
		rmAddAreaConstraint(id1, shortAvoidImpassableLand);
		rmSetAreaMinBlobs(id1, 1);
		rmSetAreaMaxBlobs(id1, 5);
		rmSetAreaWarnFailure(id1, false);
		rmSetAreaMinBlobDistance(id1, 16.0);
		rmSetAreaMaxBlobDistance(id1, 40.0);
		rmSetAreaCoherence(id1, 0.6);
		rmBuildArea(id1);
	}

	 for(z=1; <cNumberPlayers*10)
	{
		int id2=rmCreateArea("more beautification"+z);
		rmSetAreaSize(id2, rmAreaTilesToFraction(60), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id2, "EgyptianRoadA");
		rmAddAreaConstraint(id2, shortForestConstraint);
		rmAddAreaConstraint(id2, shortAvoidImpassableLand);
		rmSetAreaMinBlobs(id2, 1);
		rmSetAreaMaxBlobs(id2, 5);
		rmSetAreaWarnFailure(id2, false);
		rmSetAreaMinBlobDistance(id2, 16.0);
		rmSetAreaMaxBlobDistance(id2, 40.0);
		rmSetAreaCoherence(id2, 0.4);
		rmBuildArea(id2);
	}

	// Place close objects
	// Towers
	rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

	// Gold
	rmPlaceObjectDefPerPlayer(startingGoldID, false);

	// Hunt
	rmPlaceObjectDefPerPlayer(startingRhinoID, false);

	// Berries
	rmPlaceObjectDefPerPlayer(startingFoodID, false);

	// Pigs
	rmPlaceObjectDefPerPlayer(startingPigsID, true);

	// Text
	rmSetStatusText("",0.3);

	// Player forest 1
	failCount=0;
	numTries=cNumberPlayers;
	for(l=1; <numTries)
	{
		forestID=rmCreateArea("first player forest"+l, rmAreaID("Player"+l));
		rmSetAreaSize(forestID, rmAreaTilesToFraction(20), rmAreaTilesToFraction(20));
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaForestType(forestID, "palm forest");
		rmAddAreaConstraint(forestID, shortAvoidAll);
		rmAddAreaConstraint(forestID, shortAvoidGold);
		rmAddAreaConstraint(forestID, shortCliffConstraint);
		rmAddAreaConstraint(forestID, avoidSettlement);
		rmAddAreaConstraint(forestID, shortAvoidFood);
		rmAddAreaConstraint(forestID, shortStartingSettleConstraint);
		rmAddAreaToClass(forestID, classForest);
		rmSetAreaCoherence(forestID, 0.6);
		rmSetAreaBaseHeight(forestID, 1.0);
		rmSetAreaSmoothDistance(forestID, 2);
		rmSetAreaHeightBlend(forestID, 2);
		rmSetAreaMinBlobs(forestID, 3);
		rmSetAreaMaxBlobs(forestID, 4);
		rmSetAreaMinBlobDistance(forestID, 16.0);
		rmSetAreaMaxBlobDistance(forestID, 30.0);
		rmSetAreaCoherence(forestID, 0.0);

		if(rmBuildArea(forestID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3)
				break;
		}
		else
			failCount=0;
	}

	// Player forest 2
	failCount=0;
	numTries=cNumberPlayers;
	for(m=1; <numTries)
	{
		forestID=rmCreateArea("second player forest"+m, rmAreaID("Player"+m));
		rmSetAreaSize(forestID, rmAreaTilesToFraction(20), rmAreaTilesToFraction(20));
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaForestType(forestID, "palm forest");
		rmAddAreaConstraint(forestID, shortAvoidAll);
		rmAddAreaConstraint(forestID, shortAvoidGold);
		rmAddAreaConstraint(forestID, shortCliffConstraint);
		rmAddAreaConstraint(forestID, avoidSettlement);
		rmAddAreaConstraint(forestID, mediumForestConstraint);
		rmAddAreaConstraint(forestID, shortAvoidFood);
		rmAddAreaConstraint(forestID, shortStartingSettleConstraint);
		rmAddAreaToClass(forestID, classForest);
		rmSetAreaCoherence(forestID, 0.6);
		rmSetAreaBaseHeight(forestID, 1.0);
		rmSetAreaSmoothDistance(forestID, 2);
		rmSetAreaHeightBlend(forestID, 2);
		rmSetAreaMinBlobs(forestID, 3);
		rmSetAreaMaxBlobs(forestID, 4);
		rmSetAreaMinBlobDistance(forestID, 16.0);
		rmSetAreaMaxBlobDistance(forestID, 30.0);
		rmSetAreaCoherence(forestID, 0.0);

		if(rmBuildArea(forestID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3)
				break;
		}
		else
			failCount=0;
	}

	// Center player forest
	if(cNumberNonGaiaPlayers > 4)
	{
		failCount=0;
		for(n=1; <3)
		{
			forestID=rmCreateArea("bonus forest"+n);
			rmSetAreaSize(forestID, rmAreaTilesToFraction(60), rmAreaTilesToFraction(70));
			if(skew==1)
			{
				if(n==1)
					rmSetAreaLocation(forestID, 0.0, 0.5);
				else
					rmSetAreaLocation(forestID, 1.0, 0.5);
			}
			else
			{
				if(n==1)
					rmSetAreaLocation(forestID, 0.5, 0.0);
				else
					rmSetAreaLocation(forestID, 0.5, 1.0);
			}
			rmSetAreaWarnFailure(forestID, false);
			rmSetAreaForestType(forestID, "palm forest");
			rmSetAreaCoherence(forestID, 0.6);
			rmSetAreaBaseHeight(forestID, 1.0);
			rmSetAreaSmoothDistance(forestID, 2);
			rmSetAreaHeightBlend(forestID, 2);
			rmSetAreaMinBlobs(forestID, 3);
			rmSetAreaMaxBlobs(forestID, 4);
			rmSetAreaMinBlobDistance(forestID, 16.0);
			rmSetAreaMaxBlobDistance(forestID, 30.0);
			rmSetAreaCoherence(forestID, 0.0);

			if(rmBuildArea(forestID)==false)
			{
				// Stop trying once we fail 3 times in a row.
				failCount++;
				if(failCount==3)
					break;
			}
			else
				failCount=0;
		}
	}

	// Text
	rmSetStatusText("",0.2);

	// Straggler trees
	rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(4,6));

	// Medium objects
	// Gold
	rmPlaceObjectDefPerPlayer(mediumGoldID, false);

	// Gazelles
	if(cNumberNonGaiaPlayers == 2)
		rmPlaceObjectDefPerPlayer(gazelleID, false, 2);
	else
		rmPlaceObjectDefPerPlayer(gazelleID, false);

	// Giraffes
	rmPlaceObjectDefPerPlayer(giraffeID, false);

	// Rhinos
	rmPlaceObjectDefPerPlayer(rhinoID, false);

	// Medium Pigs
	rmPlaceObjectDefPerPlayer(mediumPigsID, false);

	// Predators
	rmPlaceObjectDefPerPlayer(predatorID, false);

	// Far objects
	// Gold
	rmPlaceObjectDefPerPlayer(farGoldID, false, 2);

	// Far Pigs
	rmPlaceObjectDefPerPlayer(farPigsID, false);

	// Relics
	rmPlaceObjectDefPerPlayer(relicID, false);

	// Random trees
	rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 25*cNumberNonGaiaPlayers);

	// Hawks
	rmPlaceObjectDefPerPlayer(farhawkID, false, 2);

	// Text
	rmSetStatusText("",0.1);

	// Decoration
	int bushID=rmCreateObjectDef("big bush patch");
	rmAddObjectDefItem(bushID, "bush", 4, 3.0);
	rmSetObjectDefMinDistance(bushID, 0.0);
	rmSetObjectDefMaxDistance(bushID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(bushID, shortAvoidAll);
	rmPlaceObjectDefAtLoc(bushID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);

	int bush2ID=rmCreateObjectDef("small bush patch");
	rmAddObjectDefItem(bush2ID, "bush", 3, 2.0);
	rmAddObjectDefItem(bush2ID, "rock sandstone sprite", 1, 2.0);
	rmSetObjectDefMinDistance(bush2ID, 0.0);
	rmSetObjectDefMaxDistance(bush2ID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(bush2ID, shortAvoidAll);
	rmPlaceObjectDefAtLoc(bush2ID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

	int grassID=rmCreateObjectDef("grass");
	rmAddObjectDefItem(grassID, "grass", 1, 0.0);
	rmSetObjectDefMinDistance(grassID, 0.0);
	rmSetObjectDefMaxDistance(grassID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(grassID, shortAvoidAll);
	rmPlaceObjectDefAtLoc(grassID, 0, 0.5, 0.5, 30*cNumberNonGaiaPlayers);

	int sandDrift=rmCreateObjectDef("blowing sand");
	rmAddObjectDefItem(sandDrift, "sand drift patch", 1, 0.0);
	rmSetObjectDefMinDistance(sandDrift, 0.0);
	rmSetObjectDefMaxDistance(sandDrift, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(sandDrift, shortAvoidAll);
	rmAddObjectDefConstraint(sandDrift, edgeConstraint);
	rmAddObjectDefConstraint(sandDrift, rmCreateTypeDistanceConstraint("drift vs drift", "sand drift dune", 25.0));
	rmAddObjectDefConstraint(sandDrift, avoidBuildings);
	rmPlaceObjectDefAtLoc(sandDrift, 0, 0.5, 0.5, 6*cNumberNonGaiaPlayers);

	int column=rmCreateObjectDef("broken columns");
	rmAddObjectDefItem(column, "columns broken", 1, 0.0);
	rmSetObjectDefMinDistance(column, 0.0);
	rmSetObjectDefMaxDistance(column, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(column, shortAvoidAll);
	rmAddObjectDefConstraint(column, edgeConstraint);
	rmAddObjectDefConstraint(column, avoidBuildings);
	rmPlaceObjectDefAtLoc(column, 0, 0.5, 0.5, 6*cNumberNonGaiaPlayers);

	// Text
	rmSetStatusText("",0.01);

	// Notify players
	if(cNumberTeams > 2)
	{
		rmCreateTrigger("team notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
	else if(cNumberNonGaiaPlayers > 6)
	{
		rmCreateTrigger("player notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
}