// Sacred Pond 1.0
// Script: RR
// Concept: Bubble

// Main entry point for random map script
void main(void)
{
	// Text
	rmSetStatusText("",1.0);

	// Set size
	int playerTiles=7500;
	int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
	rmEchoInfo("Map size="+size+"m x "+size+"m");
	rmSetMapSize(size, size);

	// Set up default water.
	rmSetSeaLevel(0.0);

	// Init map
	rmTerrainInitialize("SandD");
	rmSetLightingSet("Dawn");

	// Define classes
	int classBonusHuntable=rmDefineClass("bonus huntable");
	int classBaboon=rmDefineClass("class baboon");
	int classElev=rmDefineClass("elevation");
	int classForest=rmDefineClass("forest");
	rmDefineClass("starting settlement");
	rmDefineClass("center");

	// Constraints
	// General
	int centerConstraint=rmCreateClassDistanceConstraint("stay away from center", rmClassID("center"), 15.0);
	int farCenterConstraint=rmCreateClassDistanceConstraint("stay far away from center", rmClassID("center"), 30.0);
	int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(8), rmZTilesToFraction(8), 1.0-rmXTilesToFraction(8), 1.0-rmZTilesToFraction(8));
	int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "land", false, 10.0);
	int elevConstraint=rmCreateClassDistanceConstraint("elev avoid elev", rmClassID("elevation"), 10.0);
	int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);

	// Settlement
	int settlementAvoidsForest=rmCreateClassDistanceConstraint("settlement avoids center forest", rmClassID("forest"), 17.5);
	int settlementAvoidsShore=rmCreateTerrainDistanceConstraint("settlement avoids center lake", "water", true, 17.5);
	int settlementAvoidsSettlement=rmCreateTypeDistanceConstraint("TCs avoid TCs by long distance", "AbstractSettlement", 60.0);
	int shortAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by short distance", "AbstractSettlement", 20.0);
	int farAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TCs by long distance", "AbstractSettlement", 50.0);
	int farStartingSettleConstraint=rmCreateClassDistanceConstraint("objects avoid player TCs", rmClassID("starting settlement"), 50.0);

	// Shore
	int forestShoreMin=rmCreateTerrainDistanceConstraint("center forest min", "water", true, 8.0);
	int forestShoreMax=rmCreateTerrainMaxDistanceConstraint("center forest max", "water", true, 20.0);
	int shoreForestConstraint=rmCreateClassDistanceConstraint("forest v forest center", rmClassID("forest"), 12.5);
	int nearShore=rmCreateTerrainMaxDistanceConstraint("near shore", "land", true, 4.0);
	int nearWater=rmCreateTerrainMaxDistanceConstraint("near water", "water", true, 5.0);

	// Gold
	int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 50.0);
	int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 10.0);

	// Food
	int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 30.0);
	int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 20.0);
	int avoidFood=rmCreateTypeDistanceConstraint("avoid other food sources", "food", 30.0);
	int avoidBonusHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", classBonusHuntable, 40.0);
	int avoidHuntable=rmCreateTypeDistanceConstraint("avoid huntable", "huntable", 40.0);
	int avoidBaboon=rmCreateClassDistanceConstraint("baboon v baboon", classBaboon, 100.0);
	int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 16.0);
	int fishLand=rmCreateTerrainDistanceConstraint("fish land", "land", true, 7.0);

	// Forest
	int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 25.0);
	int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);
	int avoidForest=rmCreateClassDistanceConstraint("avoid forest", rmClassID("forest"), 10.0);

	// Buildings
	int avoidTower=rmCreateTypeDistanceConstraint("towers avoid towers", "tower", 25.0);
	int shortAvoidBuildings=rmCreateTypeDistanceConstraint("short avoid buildings", "Building", 10.0);
	int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 20.0);

	// Embellishment
	int driftVsDrift=rmCreateTypeDistanceConstraint("drift vs drift", "sand drift dune", 25.0);

	// Text
	rmSetStatusText("",0.9);

	// Define objects
	// Starting TC
	int startingSettlementID=rmCreateObjectDef("starting settlement");
	rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
	rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
	rmSetObjectDefMinDistance(startingSettlementID, 0.0);
	rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

	// Towers
	int startingTowerID=rmCreateObjectDef("Starting tower");
	rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
	rmSetObjectDefMinDistance(startingTowerID, 20.0);
	rmSetObjectDefMaxDistance(startingTowerID, 28.0);
	rmAddObjectDefConstraint(startingTowerID, avoidTower);
	rmAddObjectDefConstraint(startingTowerID, centerConstraint);

	// Gold
	int startingGoldID=rmCreateObjectDef("Starting gold");
	rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
	rmSetObjectDefMinDistance(startingGoldID, 20.0);
	rmSetObjectDefMaxDistance(startingGoldID, 20.0);
	rmAddObjectDefConstraint(startingGoldID, centerConstraint);
	rmAddObjectDefConstraint(startingGoldID, avoidAll);

	// Goats
	int closeGoatsID=rmCreateObjectDef("close goats");
	rmAddObjectDefItem(closeGoatsID, "goat", rmRandInt(2,3), 2.0);
	rmSetObjectDefMinDistance(closeGoatsID, 25.0);
	rmSetObjectDefMaxDistance(closeGoatsID, 30.0);
	rmAddObjectDefConstraint(closeGoatsID, centerConstraint);
	rmAddObjectDefConstraint(closeGoatsID, avoidAll);

	// Berries
	int closeBerriesID=rmCreateObjectDef("close Chickens");
	rmAddObjectDefItem(closeBerriesID, "berry bush", rmRandInt(7,8), 4.0);
	rmSetObjectDefMinDistance(closeBerriesID, 20.0);
	rmSetObjectDefMaxDistance(closeBerriesID, 25.0);
	rmAddObjectDefConstraint(closeBerriesID, centerConstraint);
	rmAddObjectDefConstraint(closeBerriesID, avoidAll);
	rmAddObjectDefConstraint(closeBerriesID, shortAvoidGold);

	// Starting hunt
	int closeZebraID=rmCreateObjectDef("close zebra");
	if(rmRandFloat(0,1)<0.5)
		rmAddObjectDefItem(closeZebraID, "zebra", rmRandInt(4,5), 2.0);
	else
		rmAddObjectDefItem(closeZebraID, "gazelle", rmRandInt(5,6), 2.0);
	rmSetObjectDefMinDistance(closeZebraID, 20.0);
	rmSetObjectDefMaxDistance(closeZebraID, 28.0);
	rmAddObjectDefConstraint(closeZebraID, centerConstraint);
	rmAddObjectDefConstraint(closeZebraID, shortAvoidBuildings);
	rmAddObjectDefConstraint(closeZebraID, avoidForest);
	rmAddObjectDefConstraint(closeZebraID, shortAvoidGold);

	// Straggler trees
	int stragglerTreeID=rmCreateObjectDef("straggler tree");
	rmAddObjectDefItem(stragglerTreeID, "palm", 1, 0.0);
	rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
	rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);
	rmAddObjectDefConstraint(stragglerTreeID, shortAvoidGold);

	// Medium Objects
	// gold avoids gold and Settlements
	int mediumGoldID=rmCreateObjectDef("medium gold");
	rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(mediumGoldID, 40.0);
	rmSetObjectDefMaxDistance(mediumGoldID, 60.0);
	rmAddObjectDefConstraint(mediumGoldID, avoidGold);
	rmAddObjectDefConstraint(mediumGoldID, edgeConstraint);
	rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
	rmAddObjectDefConstraint(mediumGoldID, centerConstraint);
	rmAddObjectDefConstraint(mediumGoldID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(mediumGoldID, avoidAll);

	int mediumGoatsID=rmCreateObjectDef("medium Goats");
	rmAddObjectDefItem(mediumGoatsID, "goat", 2, 4.0);
	rmSetObjectDefMinDistance(mediumGoatsID, 50.0);
	rmSetObjectDefMaxDistance(mediumGoatsID, 70.0);
	rmAddObjectDefConstraint(mediumGoatsID, centerConstraint);
	rmAddObjectDefConstraint(mediumGoatsID, avoidHerdable);
	rmAddObjectDefConstraint(mediumGoatsID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(mediumGoatsID, avoidAll);

	// Far Objects
	// gold avoids gold, Settlements and TCs
	int farGoldID=rmCreateObjectDef("far gold");
	rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
	rmSetObjectDefMinDistance(farGoldID, 80.0);
	rmSetObjectDefMaxDistance(farGoldID, 120.0);
	rmAddObjectDefConstraint(farGoldID, avoidGold);
	rmAddObjectDefConstraint(farGoldID, edgeConstraint);
	rmAddObjectDefConstraint(farGoldID, centerConstraint);
	rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
	rmAddObjectDefConstraint(farGoldID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(farGoldID, avoidAll);

	// Goats avoid TCs and other herds
	int farGoatsID=rmCreateObjectDef("far Goats");
	rmAddObjectDefItem(farGoatsID, "goat", 2, 4.0);
	rmSetObjectDefMinDistance(farGoatsID, 80.0);
	rmSetObjectDefMaxDistance(farGoatsID, 150.0);
	rmAddObjectDefConstraint(farGoatsID, centerConstraint);
	rmAddObjectDefConstraint(farGoatsID, avoidHerdable);
	rmAddObjectDefConstraint(farGoatsID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(farGoatsID, avoidAll);

	// Pick lions or bears as predators
	int farPredatorID=rmCreateObjectDef("far predator");
	if(rmRandFloat(0,1)<0.5)
		rmAddObjectDefItem(farPredatorID, "lion", 2, 4.0);
	else
		rmAddObjectDefItem(farPredatorID, "hyena", 2, 4.0);
	rmSetObjectDefMinDistance(farPredatorID, 50.0);
	rmSetObjectDefMaxDistance(farPredatorID, 100.0);
	rmAddObjectDefConstraint(farPredatorID, avoidPredator);
	rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(farPredatorID, centerConstraint);
	rmAddObjectDefConstraint(farPredatorID, avoidAll);
	rmAddObjectDefConstraint(farPredatorID, avoidFood);

	// Berries avoid TCs
	int farBerriesID=rmCreateObjectDef("far berries");
	rmAddObjectDefItem(farBerriesID, "berry bush", rmRandInt(8,10), 4.0);
	rmSetObjectDefMinDistance(farBerriesID, 20.0);
	rmSetObjectDefMaxDistance(farBerriesID, rmXFractionToMeters(0.3));
	rmAddObjectDefConstraint(farBerriesID, centerConstraint);
	rmAddObjectDefConstraint(farBerriesID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(farBerriesID, shortAvoidGold);
	rmAddObjectDefConstraint(farBerriesID, shortAvoidSettlement);
	rmAddObjectDefConstraint(farBerriesID, forestConstraint);
	rmAddObjectDefConstraint(farBerriesID, avoidFood);
	rmAddObjectDefConstraint(farBerriesID, edgeConstraint);

	// Bonus hunt avoids other hunt
	int bonusHuntableID=rmCreateObjectDef("bonus huntable");
	float bonusChance=rmRandFloat(0, 1);
	if(bonusChance<0.5)
	{
		rmAddObjectDefItem(bonusHuntableID, "giraffe", rmRandInt(2,3), 2.0);
		rmAddObjectDefItem(bonusHuntableID, "gazelle", rmRandInt(2,3), 3.0);
	}
	else
		rmAddObjectDefItem(bonusHuntableID, "giraffe", rmRandInt(3,4), 2.0);
	rmSetObjectDefMinDistance(bonusHuntableID, rmXFractionToMeters(0.1));
	rmSetObjectDefMaxDistance(bonusHuntableID, rmXFractionToMeters(0.3));
	rmAddObjectDefConstraint(bonusHuntableID, avoidBonusHuntable);
	rmAddObjectDefConstraint(bonusHuntableID, avoidHuntable);
	rmAddObjectDefToClass(bonusHuntableID, classBonusHuntable);
	rmAddObjectDefConstraint(bonusHuntableID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(bonusHuntableID, centerConstraint);
	rmAddObjectDefConstraint(bonusHuntableID, avoidAll);
	rmAddObjectDefConstraint(bonusHuntableID, avoidFood);
	rmAddObjectDefConstraint(bonusHuntableID, edgeConstraint);

	int moreBonusHuntableID=rmCreateObjectDef("more bonus huntable");
	rmAddObjectDefItem(moreBonusHuntableID, "elephant", 2, 5.0);
	rmSetObjectDefMinDistance(moreBonusHuntableID, rmXFractionToMeters(0.15));
	rmSetObjectDefMaxDistance(moreBonusHuntableID, rmXFractionToMeters(0.35));
	rmAddObjectDefConstraint(moreBonusHuntableID, avoidBonusHuntable);
	rmAddObjectDefConstraint(moreBonusHuntableID, avoidHuntable);
	rmAddObjectDefToClass(moreBonusHuntableID, classBonusHuntable);
	rmAddObjectDefConstraint(moreBonusHuntableID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(moreBonusHuntableID, centerConstraint);
	rmAddObjectDefConstraint(moreBonusHuntableID, avoidAll);
	rmAddObjectDefConstraint(moreBonusHuntableID, avoidFood);
	rmAddObjectDefConstraint(moreBonusHuntableID, edgeConstraint);

	// Baboons
	int baboonID=rmCreateObjectDef("baboons");
	rmAddObjectDefItem(baboonID, "baboon", rmRandInt(6,7), 4.0);
	rmSetObjectDefMinDistance(baboonID, rmXFractionToMeters(0.5));
	rmSetObjectDefMaxDistance(baboonID, rmXFractionToMeters(0.5));
	rmAddObjectDefToClass(baboonID, classBaboon);
	rmAddObjectDefConstraint(baboonID, avoidBaboon);
	rmAddObjectDefConstraint(baboonID, farStartingSettleConstraint);
	rmAddObjectDefConstraint(baboonID, centerConstraint);
	rmAddObjectDefConstraint(baboonID, avoidAll);
	rmAddObjectDefConstraint(baboonID, avoidFood);
	rmAddObjectDefConstraint(baboonID, edgeConstraint);

	// Player Fish
	int playerFishID=rmCreateObjectDef("owned fish");
	rmAddObjectDefItem(playerFishID, "fish - perch", 3, 8.0);
	rmSetObjectDefMinDistance(playerFishID, 0.0);
	rmSetObjectDefMaxDistance(playerFishID, rmXFractionToMeters(0.3));
	rmAddObjectDefConstraint(playerFishID, fishVsFishID);
	rmAddObjectDefConstraint(playerFishID, fishLand);

	// More Fish
	int fishID=rmCreateObjectDef("fish");
	rmAddObjectDefItem(fishID, "fish - mahi", 3, 8.0);
	rmSetObjectDefMinDistance(fishID, 0.0);
	rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(fishID, fishVsFishID);
	rmAddObjectDefConstraint(fishID, fishLand);

	int moreFishID=rmCreateObjectDef("more fish");
	rmAddObjectDefItem(moreFishID, "fish - mahi", 2, 4.0);
	rmSetObjectDefMinDistance(moreFishID, 0.0);
	rmSetObjectDefMaxDistance(moreFishID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(moreFishID, fishVsFishID);
	rmAddObjectDefConstraint(moreFishID, fishLand);

	// Random Trees
	int randomTreeID=rmCreateObjectDef("random tree");
	rmAddObjectDefItem(randomTreeID, "palm", 1, 0.0);
	rmSetObjectDefMinDistance(randomTreeID, 0.0);
	rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
	rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);
	rmAddObjectDefConstraint(randomTreeID, centerConstraint);
	rmAddObjectDefConstraint(randomTreeID, avoidAll);

	// Birds
	int farhawkID=rmCreateObjectDef("far hawks");
	rmAddObjectDefItem(farhawkID, "vulture", 1, 0.0);
	rmSetObjectDefMinDistance(farhawkID, 0.0);
	rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

	// Relics avoid TCs
	int relicID=rmCreateObjectDef("relic");
	rmAddObjectDefItem(relicID, "relic", 1, 0.0);
	rmSetObjectDefMinDistance(relicID, 60.0);
	rmSetObjectDefMaxDistance(relicID, 150.0);
	rmAddObjectDefConstraint(relicID, edgeConstraint);
	rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0));
	rmAddObjectDefConstraint(relicID, farAvoidSettlement);
	rmAddObjectDefConstraint(relicID, centerConstraint);
	rmAddObjectDefConstraint(relicID, avoidAll);

	// Text
	rmSetStatusText("",0.8);

	// Start to generate the main terrain
	// Create a center water area
	int centerID=rmCreateArea("center");
	rmSetAreaSize(centerID, 0.105, 0.105);
	rmSetAreaLocation(centerID, 0.5, 0.5);
	rmSetAreaWaterType(centerID, "Egyptian Nile");
	rmAddAreaToClass(centerID, rmClassID("center"));
	rmSetAreaBaseHeight(centerID, 0.0);
	rmSetAreaSmoothDistance(centerID, 50);
	rmSetAreaCoherence(centerID, 0.05);
	rmBuildArea(centerID);

	// Center forest
	int failCount=0;
	int numTries=8*cNumberNonGaiaPlayers;
	for(i=0; <numTries)
	{
		int forestID=rmCreateArea("forest"+i);
		rmSetAreaSize(forestID, rmAreaTilesToFraction(30+10*cNumberNonGaiaPlayers), rmAreaTilesToFraction(30+10*cNumberNonGaiaPlayers));
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaForestType(forestID, "palm forest");
		rmAddAreaConstraint(forestID, shoreForestConstraint);
	  rmAddAreaConstraint(forestID, forestShoreMax);
	  rmAddAreaConstraint(forestID, forestShoreMin);
		rmAddAreaToClass(forestID, classForest);
		rmSetAreaCoherence(forestID, 1.0);

		if(rmBuildArea(forestID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3)
				break;
		}
		else
			failCount=0;
	}

	// Text
	rmSetStatusText("",0.7);

	// Place Players
	if(cNumberNonGaiaPlayers == 2)
	{
		int startpos = rmRandInt(1,4);
	  if(startpos == 1)
	  {
			rmPlacePlayer(1, rmRandFloat(0.17,0.23), rmRandFloat(0.17,0.23));
			rmPlacePlayer(2, rmRandFloat(0.77,0.83), rmRandFloat(0.77,0.83));
		}
	  else if(startpos == 2)
	  {
			rmPlacePlayer(1, rmRandFloat(0.77,0.83), rmRandFloat(0.77,0.83));
			rmPlacePlayer(2, rmRandFloat(0.17,0.23), rmRandFloat(0.17,0.23));
		}
	  else if(startpos == 3)
	  {
			rmPlacePlayer(1, rmRandFloat(0.17,0.23), rmRandFloat(0.77,0.83));
			rmPlacePlayer(2, rmRandFloat(0.77,0.83), rmRandFloat(0.17,0.23));
		}
	  else if(startpos == 4)
	  {
			rmPlacePlayer(1, rmRandFloat(0.77,0.83), rmRandFloat(0.17,0.23));
			rmPlacePlayer(2, rmRandFloat(0.17,0.23), rmRandFloat(0.77,0.83));
		}
	}
	else
	{
		rmSetTeamSpacingModifier(0.85);
		rmPlacePlayersCircular(0.4, 0.4, rmDegreesToRadians(5.0));
	}

	// Starting TC
	rmPlaceObjectDefPerPlayer(startingSettlementID, true);

	// Settlements
	if(cNumberNonGaiaPlayers == 2)
	{
		// Settlement 1
	  int id=rmAddFairLoc("Settlement", false, true, 70, 80, 50, 16);
	  rmAddFairLocConstraint(id, settlementAvoidsForest);
		rmAddFairLocConstraint(id, settlementAvoidsShore);

		if(rmPlaceFairLocs())
		{
			id=rmCreateObjectDef("close settlement");
			rmAddObjectDefItem(id, "Settlement", 1, 0.0);
			for(i=1; <cNumberPlayers)
			{
				for(j=0; <rmGetNumberFairLocs(i))
					rmPlaceObjectDefAtLoc(id, i, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j), 1);
			}
		}

		rmResetFairLocs();

		// Settlement 2
		id=rmAddFairLoc("Settlement", true, false, 85, 115, 50, 16);
		rmAddFairLocConstraint(id, farStartingSettleConstraint);
		rmAddFairLocConstraint(id, settlementAvoidsSettlement);
		rmAddFairLocConstraint(id, settlementAvoidsForest);
		rmAddFairLocConstraint(id, settlementAvoidsShore);

		if(rmPlaceFairLocs())
		{
			id=rmCreateObjectDef("far settlement");
			rmAddObjectDefItem(id, "Settlement", 1, 0.0);
			for(i=1; <cNumberPlayers)
			{
				for(j=0; <rmGetNumberFairLocs(i))
				{
				int bonusSettleArea = rmCreateArea("bonus settlement area"+i +j);
					rmSetAreaLocation(bonusSettleArea, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j));
					rmBuildArea(bonusSettleArea);
				 rmPlaceObjectDefAtAreaLoc(id, i, bonusSettleArea);
				}
			}
		}
	}
	else
	{
		id=rmAddFairLoc("Settlement", false, true, 55, 70, 50, 16);
		id=rmAddFairLoc("Settlement", true, false, 70, 100, 50, 16);

		rmAddFairLocConstraint(id, settlementAvoidsForest);
		rmAddFairLocConstraint(id, settlementAvoidsShore);
		rmAddFairLocConstraint(id, settlementAvoidsSettlement);

		if(rmPlaceFairLocs())
		{
			id=rmCreateObjectDef("far settlement2");
			rmAddObjectDefItem(id, "Settlement", 1, 0.0);
			for(i=1; <cNumberPlayers)
			{
				for(j=0; <rmGetNumberFairLocs(i))
					rmPlaceObjectDefAtLoc(id, i, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j), 1);
			}
		}
	}

	// Text
	rmSetStatusText("",0.6);

	// Elevation
	numTries=10*cNumberNonGaiaPlayers;
	failCount=0;
	for(i=0; <numTries)
	{
		int elevID=rmCreateArea("elev"+i);
		rmSetAreaSize(elevID, rmAreaTilesToFraction(100), rmAreaTilesToFraction(200));
		rmSetAreaWarnFailure(elevID, false);
		rmAddAreaConstraint(elevID, avoidBuildings);
		rmAddAreaConstraint(elevID, centerConstraint);
		rmAddAreaConstraint(elevID, avoidImpassableLand);
		rmAddAreaConstraint(elevID, forestConstraint);
		if(rmRandFloat(0.0, 1.0)<0.5)
			rmSetAreaTerrainType(elevID, "SandD");
		rmSetAreaBaseHeight(elevID, rmRandFloat(3.0, 6.0));
		rmSetAreaHeightBlend(elevID, 2);
		rmAddAreaToClass(elevID, classElev);
		rmSetAreaMinBlobs(elevID, 1);
		rmSetAreaMaxBlobs(elevID, 5);
		rmSetAreaMinBlobDistance(elevID, 16.0);
		rmSetAreaMaxBlobDistance(elevID, 40.0);
		rmSetAreaCoherence(elevID, 0.0);

		if(rmBuildArea(elevID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==6)
				break;
		}
		else
			failCount=0;
	}

	// Wrinkles
	numTries=20*cNumberNonGaiaPlayers;
	failCount=0;
	for(i=0; <numTries)
	{
		elevID=rmCreateArea("wrinkle"+i);
		rmSetAreaSize(elevID, rmAreaTilesToFraction(15), rmAreaTilesToFraction(120));
		rmSetAreaWarnFailure(elevID, false);
		rmSetAreaBaseHeight(elevID, rmRandFloat(2.0, 3.0));
		rmSetAreaMinBlobs(elevID, 1);
		rmSetAreaMaxBlobs(elevID, 3);
		if(rmRandFloat(0.0, 1.0)<0.5)
			rmSetAreaTerrainType(elevID, "SandD");
		rmSetAreaMinBlobDistance(elevID, 16.0);
		rmSetAreaMaxBlobDistance(elevID, 20.0);
		rmSetAreaCoherence(elevID, 0.0);
		rmAddAreaToClass(elevID, classElev);
		rmAddAreaConstraint(elevID, forestConstraint);
		rmAddAreaConstraint(elevID, centerConstraint);
		rmAddAreaConstraint(elevID, avoidBuildings);
		rmAddAreaConstraint(elevID, avoidImpassableLand);
		rmAddAreaConstraint(elevID, elevConstraint);

		if(rmBuildArea(elevID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==10)
				break;
		}
		else
			failCount=0;
	}

	// Text
	rmSetStatusText("",0.5);

	// Texture embellishment
	for(i=1; <cNumberPlayers*10)
	{
		int id1=rmCreateArea("beautification"+i);
		rmSetAreaSize(id1, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id1, "EgyptianRoadA");
		rmAddAreaConstraint(id1, centerConstraint);
		rmAddAreaConstraint(id1, forestConstraint);
		rmSetAreaMinBlobs(id1, 1);
		rmSetAreaMaxBlobs(id1, 5);
		rmSetAreaWarnFailure(id1, false);
		rmSetAreaMinBlobDistance(id1, 16.0);
		rmSetAreaMaxBlobDistance(id1, 40.0);
		rmSetAreaCoherence(id1, 0.6);

		rmBuildArea(id1);
	}

	 for(i=1; <cNumberPlayers*12)
	{
		int id2=rmCreateArea("more beautification"+i);
		rmSetAreaSize(id2, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
		rmSetAreaTerrainType(id2, "SandB");
		rmAddAreaConstraint(id2, centerConstraint);
		rmAddAreaConstraint(id2, forestConstraint);
		rmSetAreaMinBlobs(id2, 1);
		rmSetAreaMaxBlobs(id2, 5);
		rmSetAreaWarnFailure(id2, false);
		rmSetAreaMinBlobDistance(id2, 16.0);
		rmSetAreaMaxBlobDistance(id2, 40.0);
		rmSetAreaCoherence(id2, 0.2);

		rmBuildArea(id2);
	}

	// Text
	rmSetStatusText("",0.4);

	// Place stuff
	// Towers
	rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

	// Straggler trees
	rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(4,7));

	// Gold
	rmPlaceObjectDefPerPlayer(startingGoldID, false);

	// Zebra
	rmPlaceObjectDefPerPlayer(closeZebraID, false);

	// Berries
	rmPlaceObjectDefPerPlayer(closeBerriesID, true);

	// Goats
	rmPlaceObjectDefPerPlayer(closeGoatsID, true);

	// Medium things
	// Gold
	rmPlaceObjectDefPerPlayer(mediumGoldID, false);

	// Goats
	for(i=1; <cNumberPlayers)
		rmPlaceObjectDefAtLoc(mediumGoatsID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 1);

	// Far things
	// Gold
	rmPlaceObjectDefPerPlayer(farGoldID, false, rmRandInt(2,3));

	// Bonus huntable
	for(i=1; <cNumberPlayers)
		rmPlaceObjectDefAtLoc(bonusHuntableID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 1);
	for(i=1; <cNumberPlayers)
		rmPlaceObjectDefAtLoc(moreBonusHuntableID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 1);

	// Berries
	for(i=1; <cNumberPlayers)
		rmPlaceObjectDefAtLoc(farBerriesID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 1);

	// Goats
	for(i=1; <cNumberPlayers)
		rmPlaceObjectDefAtLoc(farGoatsID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 1);

	// Predators
	rmPlaceObjectDefPerPlayer(farPredatorID, false, 1);

	// Baboons
	if(cNumberNonGaiaPlayers < 3)
	{
		for(i=1; <cNumberPlayers)
			rmPlaceObjectDefAtLoc(baboonID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 1);
	}

	// Fish
	rmPlaceObjectDefPerPlayer(playerFishID, false);
	rmPlaceObjectDefPerPlayer(fishID, false);
	rmPlaceObjectDefPerPlayer(moreFishID, false);

	// Text
	rmSetStatusText("",0.3);

	// Forest
	int forestCount=9*cNumberNonGaiaPlayers;
	failCount=0;
	for(i=0; <forestCount)
	{
		int forest1ID=rmCreateArea("forest0"+i);
		rmSetAreaSize(forest1ID, rmAreaTilesToFraction(25), rmAreaTilesToFraction(100));
		rmSetAreaWarnFailure(forest1ID, false);
		rmSetAreaForestType(forest1ID, "palm forest");
		rmAddAreaConstraint(forest1ID, forestSettleConstraint);
		rmAddAreaConstraint(forest1ID, avoidAll);
		rmAddAreaConstraint(forest1ID, forestConstraint);
		rmAddAreaConstraint(forest1ID, centerConstraint);
		rmAddAreaToClass(forest1ID, classForest);
		rmSetAreaMinBlobs(forest1ID, 1);
		rmSetAreaMaxBlobs(forest1ID, 4);
		rmSetAreaMinBlobDistance(forest1ID, 22.0);
		rmSetAreaMaxBlobDistance(forest1ID, 32.0);
		rmSetAreaCoherence(forest1ID, 0.1);
		if(rmBuildArea(forest1ID)==false)
		{
			// Stop trying once we fail 3 times in a row.
			failCount++;
			if(failCount==3)
				break;
		}
		else
			failCount=0;
	}

	// Relics
	rmPlaceObjectDefPerPlayer(relicID, false, 1);

	// Random trees
	rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 10*cNumberNonGaiaPlayers);

	// Hawks
	rmPlaceObjectDefPerPlayer(farhawkID, false, 2);

	// Text
	rmSetStatusText("",0.2);

	// Decoration
	int bushID=rmCreateObjectDef("big bush patch");
	rmAddObjectDefItem(bushID, "bush", 4, 3.0);
	rmSetObjectDefMinDistance(bushID, 0.0);
	rmSetObjectDefMaxDistance(bushID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(bushID, avoidAll);
	rmAddObjectDefConstraint(bushID, centerConstraint);
	rmPlaceObjectDefAtLoc(bushID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);

	int bush2ID=rmCreateObjectDef("small bush patch");
	rmAddObjectDefItem(bush2ID, "bush", 3, 2.0);
	rmAddObjectDefItem(bush2ID, "rock sandstone sprite", 1, 2.0);
	rmSetObjectDefMinDistance(bush2ID, 0.0);
	rmSetObjectDefMaxDistance(bush2ID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(bush2ID, avoidAll);
	rmAddObjectDefConstraint(bush2ID, centerConstraint);
	rmPlaceObjectDefAtLoc(bush2ID, 0, 0.5, 0.5, 3*cNumberNonGaiaPlayers);

	int grassID=rmCreateObjectDef("grass");
	rmAddObjectDefItem(grassID, "grass", 1, 0.0);
	rmSetObjectDefMinDistance(grassID, 0.0);
	rmSetObjectDefMaxDistance(grassID, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(grassID, avoidAll);
	rmAddObjectDefConstraint(grassID, centerConstraint);
	rmPlaceObjectDefAtLoc(grassID, 0, 0.5, 0.5, 30*cNumberNonGaiaPlayers);

	int sandDrift=rmCreateObjectDef("blowing sand");
	rmAddObjectDefItem(sandDrift, "sand drift patch", 1, 0.0);
	rmSetObjectDefMinDistance(sandDrift, 0.0);
	rmSetObjectDefMaxDistance(sandDrift, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(sandDrift, avoidAll);
	rmAddObjectDefConstraint(sandDrift, centerConstraint);
	rmAddObjectDefConstraint(sandDrift, edgeConstraint);
	rmAddObjectDefConstraint(sandDrift, driftVsDrift);
	rmAddObjectDefConstraint(sandDrift, avoidBuildings);
	rmPlaceObjectDefAtLoc(sandDrift, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);

	int column=rmCreateObjectDef("broken columns");
	rmAddObjectDefItem(column, "columns broken", 1, 0.0);
	rmSetObjectDefMinDistance(column, 0.0);
	rmSetObjectDefMaxDistance(column, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(column, avoidAll);
	rmAddObjectDefConstraint(column, centerConstraint);
	rmAddObjectDefConstraint(column, edgeConstraint);
	rmAddObjectDefConstraint(column, avoidBuildings);
	rmPlaceObjectDefAtLoc(column, 0, 0.5, 0.5, 6*cNumberNonGaiaPlayers);

	// Text
	rmSetStatusText("",0.1);

	// Water Decoration
	int papyrusID=rmCreateObjectDef("lone papyrus");
	rmAddObjectDefItem(papyrusID, "papyrus", 2, 2.0);
	rmSetObjectDefMinDistance(papyrusID, 0.0);
	rmSetObjectDefMaxDistance(papyrusID, 600);
	rmAddObjectDefConstraint(papyrusID, avoidAll);
	rmAddObjectDefConstraint(papyrusID, nearShore);
	rmPlaceObjectDefAtLoc(papyrusID, 0, 0.5, 0.5, 6*cNumberNonGaiaPlayers);

	int papyrus2ID=rmCreateObjectDef("grouped papyrus");
	rmAddObjectDefItem(papyrus2ID, "papyrus", 3, 7.0);
	rmSetObjectDefMinDistance(papyrus2ID, 0.0);
	rmSetObjectDefMaxDistance(papyrus2ID, 600);
	rmAddObjectDefConstraint(papyrus2ID, avoidAll);
	rmAddObjectDefConstraint(papyrus2ID, nearShore);
	rmPlaceObjectDefAtLoc(papyrus2ID, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);

	int lilyID=rmCreateObjectDef("water lilly");
	rmAddObjectDefItem(lilyID, "water lilly", 1, 0.0);
	rmSetObjectDefMinDistance(lilyID, 0.0);
	rmSetObjectDefMaxDistance(lilyID, 600);
	rmAddObjectDefConstraint(lilyID, avoidAll);
	rmPlaceObjectDefAtLoc(lilyID, 0, 0.5, 0.5, 8*cNumberNonGaiaPlayers);

	int riverBushID=rmCreateObjectDef("bushs by river");
	rmAddObjectDefItem(riverBushID, "bush", 3, 3.0);
	rmAddObjectDefItem(riverBushID, "grass", 7, 8.0);
	rmSetObjectDefMinDistance(riverBushID, 0.0);
	rmSetObjectDefMaxDistance(riverBushID, 600);
	rmAddObjectDefConstraint(riverBushID, avoidAll);
	rmAddObjectDefConstraint(riverBushID, nearWater);
	rmPlaceObjectDefAtLoc(riverBushID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);

	// Text
	rmSetStatusText("",0.01);

	// Triggers
	// Disable bolt for a minute
	int zeusPlayer=0;
	for(i=1; <cNumberPlayers)
	{
		if(rmGetPlayerCiv(i) == cCivZeus)
		{
			zeusPlayer=1;
			rmCreateTrigger("disable bolt"+i);
			rmSetTriggerActive(true);
			rmSetTriggerPriority(5);
			rmAddTriggerCondition("Timer");
			rmSetTriggerConditionParamInt("Param1", 1);
			rmAddTriggerEffect("Player Kill All God Powers");
			rmSetTriggerEffectParamInt("Player", i);

			rmCreateTrigger("grant bolt"+i);
			rmSetTriggerActive(true);
			rmSetTriggerPriority(5);
			rmAddTriggerCondition("Timer");
			rmSetTriggerConditionParamInt("Param1", 60);
			rmAddTriggerEffect("Grant God Power");
			rmSetTriggerEffectParamInt("PlayerID", i);
			rmSetTriggerEffectParam("PowerName", "bolt");
			rmSetTriggerEffectParamInt("Count", 1);
		}
	}

	// Notify players 1
	if(cNumberTeams > 2)
	{
		rmCreateTrigger("team notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
	else if(cNumberNonGaiaPlayers > 6)
	{
		rmCreateTrigger("player notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: This map has not been optimized for 4v4+ and FFA. No balance guaranteed - play at your own risk!", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}

	// Notify players 2
	if(zeusPlayer==1)
	{
		rmCreateTrigger("bolt notification");
		rmSetTriggerPriority(5);
		rmSetTriggerActive(true);
		rmAddTriggerCondition("Timer");
		rmAddTriggerEffect("Send Spoofed Chat");
		rmSetTriggerEffectParam("Message", "RebelsRising: Zeus' Bolt power has been disabled for the first minute.", false);
		rmSetTriggerEffectParam("PlayerID", ""+0);
	}
}